/*! @file CacheManager.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */
import Foundation
import VaultMobileSDK

class CacheManager {
    
    static let sharedInstance = CacheManager()
    var documentsCache = [UInt : VEVVaultJSONCache]()
    var cacheType = VEVVaultJSONCache.self
    let nameField = VEVVaultJSONField("name__v", type: .text)
    let idField = VEVVaultJSONField("id", type: .integer)
    var downloadsInProgress = [Int : Bool]()
    var delegate : CacheManagerDelegate?
    
    func isDocumentFileCached(documentId : Int?, vault : VEVVault?, user : VEVUser?) -> Bool {
        guard let documentId = documentId, let vault = vault, let path = CacheManager.sharedInstance.getDocumentFileURL(documentId: documentId, vault: vault, user: user) else {
            assert(false, "Invalid DocumentId, Vault or File Path")
            return false
        }
        return FileManager.default.fileExists(atPath: path.path)
    }
    
    func createCacheDirectory(username : String?){
        guard let currentUserName = username else {return}
        let path = CacheManager.getSavedDocumentsDirectory(username: currentUserName).path
        do {
            try FileManager.default.createDirectory(atPath: path, withIntermediateDirectories: true, attributes: nil)
        } catch {
            assert(false, "Unable to create cache directory \(error)")
        }
    }
    
    func getDocumentFileURL(documentId : Int?, vault : VEVVault?, user : VEVUser?) -> URL? {
        guard let documentId = documentId, let vault = vault, let currentUser = user else {
            assert(false, "Invalid DocumentId, Vault or User")
            return nil
        }
        return CacheManager.getDocumentFileURL(docID: documentId, vault: vault, user: currentUser)
    }
    
    func getCachedDocument(documentId : Int, vault : VEVVault, handler : @escaping VEVVaultJSONCacheQueryResultHandler) {
        setupCache(vault, cache: cacheType)
        let localQuery = VEVVaultLocalQuery.create(withExpression: [VEVVaultQueryCriteria(field: "id", equalTo: documentId)]);
        documentsCache[vault.id]?.query(localQuery, handler: handler)
    }
    
    func query(localQuery : VEVVaultLocalQuery, vault : VEVVault, handler : @escaping VEVVaultJSONCacheQueryResultHandler) {
        setupCache(vault, cache: cacheType)
        documentsCache[vault.id]?.query(localQuery, handler: handler)
    }
    
    fileprivate func setupCache(_ vault: VEVVault, cache : VEVVaultJSONCache.Type) {
        if documentsCache[vault.id] == nil {
            documentsCache[vault.id] = cache.open(withName: "Documents", vault: vault, fields: [nameField, idField])
        }
    }
    
    fileprivate func handleError(_ errorDescription : String ,_ error: Error?) {
        if let error = error {
            let errorString = "\(errorDescription) \(error.localizedDescription)"
            print(errorString)
            assert(false, errorString)
        }
    }

    static func getUserDirectory(username : String) -> URL {
        var userDir = URL(fileURLWithPath: NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0])
        userDir.appendPathComponent(username)
        return userDir
    }
    
    static func getSavedDocumentsDirectory(username : String) -> URL {
        var savedDocumentsDirectory = getUserDirectory(username: username)
        savedDocumentsDirectory.appendPathComponent("SavedDocuments")
        return savedDocumentsDirectory
    }

    static func getDocumentFileURL(docID : Int, vault : VEVVault, user : VEVUser) -> URL {
        let savedDocumentsDirectory = getSavedDocumentsDirectory(username: user.username)
        return savedDocumentsDirectory.appendingPathComponent("document\(docID)v\(vault.id).pdf")
    }
    
    static func deleteDocumentFile(docId : Int, vault : VEVVault, user : VEVUser?) -> Bool {
        guard let user = user else  {
            return false
        }
        let documentFileURL = getDocumentFileURL(docID: docId, vault: vault, user: user)
        do {
            try FileManager.default.removeItem(at: documentFileURL)
            return true
        } catch {
            return false
        }
    }

}

//Remove Document from Cache
extension CacheManager {
    func removeDocumentFromCache(documentId : Int?, vault : VEVVault?){
        guard let documentId = documentId, let vault = vault else {
            assert(false, "Invalid DocumentId or Vault")
            delegate?.fileRemovalFinished(for: nil, error:  NSError(domain: kCacheErrorDomain, code: CacheError.removalFailed, userInfo: nil))
            return
        }
        setupCache(vault, cache: cacheType)
        
        getCachedDocument(documentId: documentId, vault: vault) { [weak self] (entries, error) in
            self?.deleteCachedEntries(documentId, vault, entries, error)
        }
    }
    
    fileprivate func deleteCachedEntries(_ documentId : Int, _ vault : VEVVault, _ entries: [VEVVaultJSONCacheEntry]? , _ error: Error?) {
        if let error = error {
            handleError("Error occurred removing document from cache",error)
            delegate?.fileRemovalFinished(for: documentId, error: error)
            return
        }
        if let entries = entries {
            removeEntryFromCache(documentId, vault, entries)
        }
    }
    
    fileprivate func removeEntryFromCache(_ documentId : Int, _ vault : VEVVault, _ entries: [VEVVaultJSONCacheEntry]) {
        documentsCache[vault.id]?.removeEntries(entries, handler: { [weak self] (error) in
            if let error = error {
                self?.removeEntryFailure(documentId, error: error)
            } else {
                self?.removeEntrySuccess(documentId, vault)
            }
            return
        })
    }
    
    fileprivate func removeEntryFailure(_ documentId : Int, error : Error ){
        handleError("Error occurred removing entries from cache",error)
        delegate?.fileRemovalFinished(for: documentId, error: error)
    }
    
    fileprivate func removeEntrySuccess(_ documentId : Int, _ vault : VEVVault) {
        if CacheManager.deleteDocumentFile(docId: documentId, vault: vault, user: VEVVaultAuth.sharedInstance().currentUser) {
            delegate?.fileRemovalFinished(for: documentId, error: nil)
        } else {
            delegate?.fileRemovalFinished(for: documentId, error:  NSError(domain: kCacheErrorDomain, code: CacheError.removalFailed, userInfo: nil))
        }
    }
}

//Cache Document
extension CacheManager {
    
    func downloadAndCacheDocumentFile(_ vault: VEVVault?, _ documentMetadata:  DocumentMetadata?, _ user : VEVUser?) {
        if let documentId = documentMetadata?.vaultDocumentId, let vault = vault, let fileURL = getDocumentFileURL(documentId: documentId, vault: vault, user: user ), let documentMetadata = documentMetadata, downloadsInProgress[documentId] == nil {
            downloadsInProgress[documentId] = true
            invokeVaultEndpointToDownloadDocument(vault, documentId, fileURL, documentMetadata) { [weak self] (_, _, _, _, error) in
                self?.cacheDocumentResponse(documentId: documentId, vault: vault, documentMetadata: documentMetadata, error: error)
            }
            
        } else {
            delegate?.fileCacheFinished(for: documentMetadata?.vaultDocumentId, error : NSError(domain: kCacheErrorDomain, code: CacheError.cacheFailed, userInfo: nil))
        }
    }
    
    @objc func invokeVaultEndpointToDownloadDocument(_ vault: VEVVault, _ documentId: Int, _ fileURL: URL, _ documentMetadata: DocumentMetadata, completion : @escaping VEVVaultAPIDownloadResultHandler) {
        VaultAPI.downloadDocument(from: vault, documentId: documentId, fileURL: fileURL , progressHandler: nil, completion: completion)
    }
    
    func cacheDocumentResponse(documentId : Int, vault : VEVVault, documentMetadata : DocumentMetadata, error : Error? ){
        downloadsInProgress[documentId] = nil
        if error == nil {
            cacheResponse(response: documentMetadata.getRawResponse() , vault: vault)
        }
        delegate?.fileCacheFinished(for: documentId, error: error)
    }
    
    fileprivate func cacheResponse(response : Dictionary<String, Any>, vault : VEVVault ){
        setupCache(vault, cache: cacheType)
        documentsCache[vault.id]?.cache([response], vault: vault)
    }
    
}
 
 protocol CacheManagerDelegate {
    func fileCacheFinished(for documentId : Int? , error : Error?)
    func fileRemovalFinished(for documentId : Int? , error : Error?)
    
 }
