/*! @file DocumentDetailViewController.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

import UIKit
import PDFReader
import VaultMobileSDK

class DocumentDetailViewController: UIViewController {
    @IBOutlet weak var removeButton: UIBarButtonItem!
    @IBAction func removeButtonPressed(_ sender: Any) {
        CacheManager.sharedInstance.removeDocumentFromCache(documentId: viewModel.documentMetadata.vaultDocumentId, vault: viewModel.vault)
    }
    
    @IBOutlet weak var saveButton: UIBarButtonItem!
    @IBAction func saveButtonPressed(_ sender: Any) {
        CacheManager.sharedInstance.downloadAndCacheDocumentFile(viewModel.vault, viewModel.documentMetadata, VEVVaultAuth.sharedInstance().currentUser)
        tableView.reloadRows(at: [IndexPath(row: 0, section: 0)], with: .automatic)
    }
    
    @IBAction func viewButtonPressed(_ sender: Any) {
        presentPDFDocument()
    }
    
    @IBOutlet weak var tableView: UITableView!
    var viewModel : DocumentDetailViewModel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupTableView()
        navigationController?.navigationBar.tintColor = UIColor.veevaOrange
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        CacheManager.sharedInstance.delegate = self
        setButtonState()
    }
    
    fileprivate func setButtonState() {
        if CacheManager.sharedInstance.isDocumentFileCached(documentId: viewModel.documentMetadata.vaultDocumentId, vault: viewModel.vault, user: VEVVaultAuth.sharedInstance().currentUser)   {
            setButtonState(saveButtonEnabled: false, removeButtonEnabled: true, viewButtonVisible: true)
        } else if CacheManager.sharedInstance.downloadsInProgress[viewModel.documentMetadata.vaultDocumentId ?? -1] == true {
            setButtonState(saveButtonEnabled: false, removeButtonEnabled: false, viewButtonVisible: false)
        } else {
            setButtonState(saveButtonEnabled: true, removeButtonEnabled: false, viewButtonVisible: false)
        }
    }
    
    fileprivate func setButtonState(saveButtonEnabled : Bool, removeButtonEnabled : Bool, viewButtonVisible : Bool) {
        saveButton.isEnabled = saveButtonEnabled
        removeButton.isEnabled = removeButtonEnabled
        setViewFileButton(visible: viewButtonVisible)
    }
    
    fileprivate func setViewFileButton(visible : Bool) {
        if visible {
            addViewButtonToNavigationBar()
        } else {
            navigationItem.rightBarButtonItem = nil
        }
    }
    
    fileprivate func addViewButtonToNavigationBar(){
        navigationItem.rightBarButtonItem = UIBarButtonItem(title: "View", style: .plain, target: self, action: #selector(DocumentDetailViewController.viewButtonPressed(_:)))
        navigationItem.rightBarButtonItem?.setTitleTextAttributes([kCTFontAttributeName as NSAttributedStringKey:UIFont.systemFont(ofSize: 17, weight: UIFont.Weight.bold)], for: UIControlState.normal)
    }

    fileprivate func setupTableView() {
        tableView.estimatedRowHeight = 60
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.tableFooterView = UIView()
        registerTableViewCells()
    }
    
    fileprivate func registerTableViewCells() {
        tableView.register(UINib(nibName: "DocumentDetailPreviewCell", bundle: nil), forCellReuseIdentifier: "DocumentDetailPreviewCell")
        tableView.register(UINib(nibName: "DocumentDetailMetadataCell", bundle: nil), forCellReuseIdentifier: "DocumentDetailMetadataCell")
        tableView.register(UINib(nibName: "DocumentDetailVersionCell", bundle: nil), forCellReuseIdentifier: "DocumentDetailVersionCell")
        tableView.register(UINib(nibName: "DocumentDetailHeaderCell", bundle: nil), forCellReuseIdentifier: "DocumentDetailHeaderCell")
    }
    

}

extension DocumentDetailViewController : UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if indexPath.section == 0 {
            return setupFirstSection(indexPath, tableView)
        } else {
            return setupFileInfoCell(tableView, indexPath)
        }
    }
    
    fileprivate func setupFirstSection(_ indexPath: IndexPath, _ tableView: UITableView) -> UITableViewCell {
        if indexPath.row == 0 {
            return setupFilePreviewImageCell(tableView)
        } else {
            return setupDocumentMetadataCell(tableView, indexPath)
        }
    }
    
    fileprivate func setupFilePreviewImageCell(_ tableView: UITableView) -> UITableViewCell {
        let cell : DocumentDetailPreviewCell = tableView.dequeueReusableCell(withIdentifier: "DocumentDetailPreviewCell") as! DocumentDetailPreviewCell
        cell.configureCell(document: viewModel.model.documentMetadata, vault: viewModel.vault)
        return cell
    }

    fileprivate func setupDocumentMetadataCell(_ tableView: UITableView, _ indexPath: IndexPath) -> UITableViewCell {
        let cell : DocumentDetailMetadataCell = tableView.dequeueReusableCell(withIdentifier: "DocumentDetailMetadataCell") as! DocumentDetailMetadataCell
        cell.configure(response: viewModel.configurationForDocumentInfoSection(row : indexPath.row))
        return cell
    }
    
    fileprivate func setupFileInfoCell(_ tableView: UITableView, _ indexPath: IndexPath) -> UITableViewCell {
        let cell : DocumentDetailMetadataCell = tableView.dequeueReusableCell(withIdentifier: "DocumentDetailMetadataCell") as! DocumentDetailMetadataCell
        cell.configure(response: viewModel.configurationForFileInfoSection(row : indexPath.row))
        return cell
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return viewModel.numberOfRowsForSection(section: section)
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }
    
    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return section == 1 ? "File Info" : nil
    }
    
    fileprivate func presentPDFDocument() {
        if let document = viewModel.getPDFDocument() {
            let readerController = PDFViewController.createNew(with: document, title: viewModel.documentMetadata.filename, actionButtonImage: UIImage(), actionStyle:                PDFViewController.ActionStyle.customAction({}), backButton: nil, isThumbnailsEnabled: true, startPageIndex: 0  )
            navigationController?.pushViewController(readerController, animated: true)
        }
    }
    
}

extension DocumentDetailViewController : CacheManagerDelegate {
    func fileCacheFinished(for documentId: Int?, error: Error?) {
        refreshDocumentImage(documentId)
    }
    
    func fileRemovalFinished(for documentId: Int?, error: Error?) {
        refreshDocumentImage(documentId)
    }

    fileprivate func refreshDocumentImage(_ documentId: Int?) {
        setButtonState()
        if viewModel.documentMetadata.vaultDocumentId == documentId {
            tableView.reloadRows(at: [IndexPath(row: 0, section: 0)], with: .automatic)
        }
    }
}
