/*! @file DocumentDetailViewModel.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

import Foundation
import VaultMobileSDK
import PDFReader
class DocumentDetailViewModel : NSObject {
    var model : DocumentDetailModel!
    
    var documentMetadata : DocumentMetadata {
        return model.documentMetadata
    }
    
    var vault : VEVVault {
        return model.vault
    }
    
    init(model : DocumentDetailModel){
        super.init()
        self.model = model
    }

    func numberOfRowsForSection(section : Int) -> Int {
        return section == 0 ? 6 : 2
    }
    
    func configurationForDocumentInfoSection(row : Int) -> DocumentDetailCellConfigurator {
        switch row {
            case 1:
                return DocumentDetailCellConfigurator(title: "Name", subtitle: documentMetadata.name ?? " ")
            case 2:
                return DocumentDetailCellConfigurator(title: "Latest Version", subtitle: "\(documentMetadata.majorVersion ?? 0).\(documentMetadata.minorVersion ?? 0)")
            case 3:
                return DocumentDetailCellConfigurator(title: "Type", subtitle: createClassificationString() )
            case 4:
                return DocumentDetailCellConfigurator(title: "Document Number", subtitle: documentMetadata.documentNumber ?? " ")
            case 5:
                return DocumentDetailCellConfigurator(title: "Creation Date", subtitle: documentMetadata.creationDate ?? " ")
            default:
                return DocumentDetailCellConfigurator(title: "invalid", subtitle:  "invalid")
        }
    }
    
    fileprivate func createClassificationString() -> String {
        var classificationString = documentMetadata.type ?? " "
        if let subtype = documentMetadata.subtype {
            classificationString += ">\(subtype)"
        }
        if let classification = documentMetadata.classification {
            classificationString += ">\(classification)"
        }
        return classificationString
    }
    
    func configurationForFileInfoSection(row : Int) -> DocumentDetailCellConfigurator {
        switch row {
        case 0:
            return DocumentDetailCellConfigurator(title: "File name", subtitle: documentMetadata.filename ?? " ")
        case 1:
            let fileSizeWithUnit = ByteCountFormatter.string(fromByteCount: Int64(documentMetadata.filesize ?? 0), countStyle: .binary)
            return DocumentDetailCellConfigurator(title: "Size", subtitle: "\(fileSizeWithUnit)")
        default:
            return DocumentDetailCellConfigurator(title: "invalid", subtitle:  "invalid")
        }
    }
    
    func getCurrentDocumentURL() -> URL? {
        return CacheManager.sharedInstance.getDocumentFileURL(documentId: documentMetadata.vaultDocumentId, vault: vault, user: VEVVaultAuth.sharedInstance().currentUser)
    }
    
    func getPDFDocument() -> PDFDocument? {
        if let url = getCurrentDocumentURL(), let filename = documentMetadata.filename, let pdfDecrypted = try? VEVVaultCrypt.sharedInstance().decryptContents(atPath: url.path), let document = PDFDocument(fileData: pdfDecrypted, fileName: filename) {
            return document
        } else {
            return nil
        }
    }

}

struct DocumentDetailCellConfigurator {
    var title : String
    var subtitle : String
    init(title : String, subtitle : String) {
        self.title = title
        self.subtitle = subtitle
    }
}

struct DocumentVersionCellConfigurator {
    var version : String
    var url : String
    
    init(version : String, url : String) {
        self.version = version
        self.url = url
    }
}
