/*! @file DocumentListViewController.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

import UIKit
import SideMenu
import VaultMobileSDK

class DocumentsListViewController: UIViewController {
    @IBOutlet weak var searchBar: UISearchBar!
    @IBOutlet weak var documentsTableView: UITableView!
    
    @IBOutlet weak var offlineTableViewContent: UILabel!
    @IBOutlet weak var vaultTitleLabel: UILabel!
    @IBOutlet weak var topTableViewConstraint: NSLayoutConstraint!
    
    @IBOutlet weak var tabBar: UITabBar!
    
    @IBOutlet weak var serverLocalControl: UISegmentedControl!
    @IBAction func serverLocalSwitch(_ sender: Any) {
        viewModel.currentMode = viewModel.currentMode == .SavedLocally ? .ServerMode : .SavedLocally
        refreshDocumentList()
    }
    
    var keyboardDismissRecognizer : UITapGestureRecognizer?
    
    var viewModel : DocumentsListViewModel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
        viewModel.updateCurrentView(tabBarItem: VaultTabBarItem.Browse)
        refreshDocumentList()
        
        setupTabBar()
        
        setupMenuButton()
        addKeyboardObservers()
        vaultTitleLabel.text = viewModel.vault.name
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        CacheManager.sharedInstance.delegate = self
        refreshDocumentList()
    }
    
    fileprivate func setupTabBar() {
        tabBar.delegate = self
        tabBar.selectedItem = tabBar.items?[viewModel.currentTabBarSelection.rawValue]
    }
    
    
    fileprivate func setupMenuButton() {
        if let menuItemImage = UIImage(named: "More Button")?.withRenderingMode(.alwaysOriginal) {
            let button = UIBarButtonItem(image: menuItemImage, style: .plain, target: self, action:  #selector(DocumentsListViewController.moreButtonPressed))
            navigationItem.rightBarButtonItem = button
        }
    }
    
    fileprivate func presentMenuAlertController() {
        let alertController = UIAlertController(title: nil, message: nil, preferredStyle: .actionSheet)
        
        addAlertActionToController(alertController: alertController, title: "Switch Vault", style: .default) {
            UIRouter.navigateToVaultPicker(fromVC: self, currentVault: self.viewModel.vault, vaultUpdatesDelegate: self)
        }
        addAlertActionToController(alertController: alertController, title: "Log Off", style: .default) {
            VEVVaultAuth.sharedInstance().logout()
        }
        addAlertActionToController(alertController: alertController, title: "Cancel", style: .cancel) {
            alertController.dismiss(animated: true, completion: nil)
        }
        
        present(alertController, animated: true, completion: nil)
    }
    
    fileprivate func addAlertActionToController(alertController : UIAlertController, title : String, style : UIAlertActionStyle, completion : @escaping (() -> ()) ) {
        let action = UIAlertAction(title: title, style: style) { (_) in
            completion()
        }
        alertController.addAction(action)
    }
    
    @objc func moreButtonPressed(){
        presentMenuAlertController()
    }
    
    @objc func hideKeyboard() {
        view.endEditing(true)
    }
    
    fileprivate func setupUI() {
        setupTableView()
        setupNavigationController()
    }
    
    fileprivate func setupTableView() {
        documentsTableView.register(UINib(nibName: "DocumentListCell", bundle: nil), forCellReuseIdentifier: "DocumentListCell")
        documentsTableView.register(UINib(nibName: "DocumentListLoadingCell", bundle: nil), forCellReuseIdentifier: "DocumentListLoadingCell")
        documentsTableView.estimatedRowHeight = 60
        documentsTableView.rowHeight = UITableViewAutomaticDimension
        documentsTableView.tableFooterView = UIView()
    }
    
    fileprivate func setupNavigationController(){
        navigationController?.navigationBar.setBackgroundImage(UIImage(), for: UIBarMetrics.default)
        navigationController?.navigationBar.shadowImage = UIImage()
    }
    
    @objc fileprivate func presentSideMenu(button : UIBarButtonItem) {
        present(SideMenuManager.default.menuLeftNavigationController!, animated: true, completion: nil)
    }
    
}

extension DocumentsListViewController {
    
    fileprivate func addKeyboardObservers() {
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: NSNotification.Name.UIKeyboardWillShow, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: NSNotification.Name.UIKeyboardWillHide, object: nil)
    }
    
    @objc func keyboardWillShow(notification: NSNotification) {
        setupKeyboardDismissRecognizer()
    }
    
    @objc func keyboardWillHide(notification: NSNotification) {
        removeKeyboardDismissRecognizer()
    }
    
    fileprivate func removeKeyboardDismissRecognizer() {
        if let dismissRecognizer = keyboardDismissRecognizer {
            view.removeGestureRecognizer(dismissRecognizer)
        }
    }
    
    fileprivate func setupKeyboardDismissRecognizer() {
        keyboardDismissRecognizer = UITapGestureRecognizer(target: self, action: #selector(DocumentsListViewController.hideKeyboard))
        keyboardDismissRecognizer?.cancelsTouchesInView = true
        if let dismissRecognizer = keyboardDismissRecognizer {
            view.addGestureRecognizer(dismissRecognizer)
        }
    }
}

extension DocumentsListViewController : UITableViewDataSource, UITableViewDelegate {
    
    
    func tableView(_ tableView: UITableView, trailingSwipeActionsConfigurationForRowAt indexPath: IndexPath) -> UISwipeActionsConfiguration? {
        
        guard let documentMetadata = viewModel.getDocumentDataForResponseAtRow(row: indexPath.row) else { return nil }
        
        let action = createSwipeAction(documentMetadata, indexPath)
        return UISwipeActionsConfiguration(actions: [action])
    }
    
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let documentResponse = viewModel.getDocumentDataForResponseAtRow(row: indexPath.row) {
            UIRouter.navigateToDocumentDetail(fromVC: self, vault: viewModel.vault, document: documentResponse)
        }
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return viewModel.numberOfResponses(section: section)
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if let documentResponse = viewModel.getDocumentDataForResponseAtRow(row: indexPath.row) {
            return configureDocumentsCell(tableView, documentResponse)
        } else {
            return configureLoadingCell(tableView)
        }
    }
    
    fileprivate func configureLoadingCell(_ tableView : UITableView) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "DocumentListLoadingCell") as! DocumentListLoadingCell
        return cell
    }
    
    fileprivate func configureDocumentsCell(_ tableView: UITableView, _ documentResponse: DocumentMetadata) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "DocumentListCell") as! DocumentListCell
        cell.configureCell(document: documentResponse, vault: viewModel.vault)
        return cell
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        //This is used for pagination purposes
        if viewModel.shouldAttemptToFetchNewDocumentsFromVault(indexPath: indexPath) {
            queryVaultForNextSetOfDocuments(indexPath: indexPath)
        }
    }
    
    fileprivate func queryVaultForNextSetOfDocuments(indexPath : IndexPath){
        viewModel.serverQuery?.parameters?["start"] = NSNumber(value: indexPath.row)
        viewModel.executeCurrentServerQuery { [weak self] (error) in
            self?.handleResponse(error)
        }
    }
    
    fileprivate func handleResponse(_ error: Error?) {
        if let error = error {
            handleQueryError(error)
            return
        } else {
            offlineTableViewContent.isHidden = true
        }
        documentsTableView.reloadData()
        
    }
    
    fileprivate func handleQueryError(_ error: Error) {
        if isQueryMismatch(error) || isInvalidQuery(error)  {
            return
        }
        offlineTableViewContent.isHidden = false
        documentsTableView.reloadData()
        print("A server error occurred \(error.localizedDescription)")
    }

    fileprivate func isInvalidQuery(_ error : Error ) -> Bool {
         return (error as NSError).domain == kServerQueryErrorDomain && (error as NSError).code == ServerQueryError.invalidQueryObject
    }
    fileprivate func isQueryMismatch(_ error: Error) -> Bool {
        return (error as NSError).domain == kServerQueryErrorDomain && (error as NSError).code == ServerQueryError.queryMismatch
    }
    
    fileprivate func resetServerMode() {
        viewModel.executeCurrentServerQuery(completion: { [weak self] (error)  in
            if let error = error {
                print("An error occurred while refreshing document list \(error.localizedDescription)")
                self?.offlineTableViewContent.isHidden = false
            } else {
                self?.offlineTableViewContent.isHidden = true
            }
            
            self?.reloadTableViewWithAnimation()
        })
    }
    
    fileprivate func resetLocalMode() {
        viewModel.executeCurrentLocalQuery { (error) in
            if let error = error {
                print("An error occurred in the local search \(error.localizedDescription)")
            } else {
                self.reloadTableViewWithAnimation()
            }
        }
    }
    
    fileprivate func removeCurrentDocumentsInList() {
        viewModel.clearResponses()
        viewModel.model.showLoadingCell = true
        reloadTableViewWithAnimation()
    }
    
    fileprivate func refreshDocumentList() {
        removeCurrentDocumentsInList()
        if viewModel.currentMode == .SavedLocally {
            offlineTableViewContent.isHidden = true
            resetLocalMode()
        } else {
            resetServerMode()
        }
    }
    
    fileprivate func reloadTableViewWithAnimation() {
        UIView.transition(with: documentsTableView,
                          duration: 0.35,
                          options: .transitionCrossDissolve,
                          animations: { self.documentsTableView.reloadData() })
    }
}

extension DocumentsListViewController : UISearchBarDelegate {
    
    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        if searchText.count == 0 {
            removeSearchResultsAndHideKeyboard(searchBar)
        }
    }
    
    fileprivate func removeSearchResultsAndHideKeyboard(_ searchBar: UISearchBar) {
        removeCurrentDocumentsInList()
        search(nil)
        DispatchQueue.main.asyncAfter(deadline: .now() + .seconds(0), execute: {
            searchBar.endEditing(true)
        })
    }

    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        performSearchWithContentOfSearchBar(searchBar)
        searchBar.endEditing(true)
    }
    
    fileprivate func performSearchWithContentOfSearchBar(_ searchBar: UISearchBar) {
        removeCurrentDocumentsInList()
        if let searchText = searchBar.text {
            search(searchText)
        }
    }

    fileprivate func search(_ searchText: String?) {
        viewModel.performSearch(searchText: searchText) { [weak self] (error) in
            if let error = error {
                print("An error occurred while searching \(error)")
            } else {
                self?.reloadTableViewWithAnimation()
            }
        }
    }
    
}

extension DocumentsListViewController : UITabBarDelegate {
    fileprivate func hideSearch() {
        updateAndAnimateSearchBarUI(topTableViewConstraintConstant: 0, serverLocalControlAlpha: 0, searchBarAlpha: 0)
    }
    
    fileprivate func showSearch() {
        updateAndAnimateSearchBarUI(topTableViewConstraintConstant: 82, serverLocalControlAlpha: 1.0, searchBarAlpha: 1.0)
    }
    
    fileprivate func updateAndAnimateSearchBarUI(topTableViewConstraintConstant : CGFloat, serverLocalControlAlpha : CGFloat, searchBarAlpha : CGFloat) {
        topTableViewConstraint.constant = topTableViewConstraintConstant
        UIView.animate(withDuration: 0.5) {
            self.serverLocalControl.alpha = serverLocalControlAlpha
            self.searchBar.alpha = searchBarAlpha
            self.view.layoutIfNeeded()
        }
    }
    
    fileprivate func switchTabBarView(_ item: UITabBarItem) {
        switch item.tag {
        case VaultTabBarItem.Browse.rawValue:
            viewModel.updateCurrentView(tabBarItem: VaultTabBarItem.Browse)
            showSearch()
        case VaultTabBarItem.Recents.rawValue:
            viewModel.updateCurrentView(tabBarItem: VaultTabBarItem.Recents)
            hideSearch()
        case VaultTabBarItem.MyDocuments.rawValue:
            viewModel.updateCurrentView(tabBarItem: VaultTabBarItem.MyDocuments)
            hideSearch()
        case VaultTabBarItem.Favorites.rawValue:
            viewModel.updateCurrentView(tabBarItem: VaultTabBarItem.Favorites)
            hideSearch()
        default:
            assert(false, "Invalid tab bar item")
        }
    }
    
    func tabBar(_ tabBar: UITabBar, didSelect item: UITabBarItem) {
        if item.tag == viewModel.currentTabBarSelection.rawValue {return}
        changeDocumentsView(item)
    }
    
    fileprivate func changeDocumentsView(_ item: UITabBarItem) {
        serverLocalControl.selectedSegmentIndex = 0
        viewModel.currentMode = .ServerMode
        viewModel.clearResponses()
        documentsTableView.reloadData()
        searchBar.text = nil
        switchTabBarView(item)
        refreshDocumentList()
    }
    
}

extension DocumentsListViewController : VaultUpdatesDelegate {
    func vaultUpdated(newVault: VEVVault) {
        changeCurrentVault(newVault)
    }
    
    fileprivate func changeCurrentVault(_ newVault: VEVVault) {
        viewModel.vault = newVault
        vaultTitleLabel.text = newVault.name
        viewModel.updateCurrentView(tabBarItem: VaultTabBarItem.Browse)
        tabBar.selectedItem = tabBar.items?[viewModel.currentTabBarSelection.rawValue]
        refreshDocumentList()
    }
}

extension DocumentsListViewController : CacheManagerDelegate {
    func fileCacheFinished(for documentId: Int?, error: Error?) {
        documentsTableView.reloadData()
    }
    
    func fileRemovalFinished(for documentId: Int?, error: Error?) {
        refreshDocumentList()
    }
    
}

//Handle Swipe Actions
extension DocumentsListViewController {
    
    fileprivate func createSwipeAction(_ documentMetadata: DocumentMetadata, _ indexPath: IndexPath) -> UIContextualAction {
        if  CacheManager.sharedInstance.isDocumentFileCached(documentId: documentMetadata.vaultDocumentId, vault: viewModel.vault, user: VEVVaultAuth.sharedInstance().currentUser) {
            return createDeleteAction(documentMetadata)
        } else {
            return createDownloadAction(documentMetadata: documentMetadata, vault: viewModel.vault, indexPath : indexPath)
        }
    }
    
    fileprivate func createDownloadAction(documentMetadata : DocumentMetadata, vault : VEVVault, indexPath : IndexPath) -> UIContextualAction{
        
        let downloadAction = UIContextualAction(style: .normal, title: nil) { (contextAction: UIContextualAction, sourceView: UIView, completionHandler: @escaping (Bool) -> Void) in
            self.handleDownloadAction(documentMetadata ,completionHandler)
        }
        
        downloadAction.backgroundColor = UIColor(red: 0/255, green: 122/255, blue: 255/255, alpha: 1)
        downloadAction.image = UIImage(named: "Download Icon")
        return downloadAction
    }
    
    fileprivate func handleDownloadAction(_ documentMetadata : DocumentMetadata, _ completionHandler: (Bool) -> Void) {
        CacheManager.sharedInstance.downloadAndCacheDocumentFile(viewModel.vault, documentMetadata, VEVVaultAuth.sharedInstance().currentUser)
        documentsTableView.reloadData()
        completionHandler(true)
    }
    
    fileprivate func createDeleteAction(_ documentMetadata: DocumentMetadata) -> UIContextualAction {
        let deleteAction = UIContextualAction(style: .normal,
                                              title: nil) { (contextAction: UIContextualAction, sourceView: UIView, completionHandler: @escaping (Bool) -> Void) in
                                                self.handleDeleteAction(documentMetadata, completionHandler)
                                                
        }
        deleteAction.backgroundColor = UIColor(red: 255/255, green: 59/255, blue: 48/255, alpha: 1)
        deleteAction.image = UIImage(named: "Delete Icon")
        return deleteAction
    }
    
    fileprivate func handleDeleteAction(_ documentMetadata : DocumentMetadata, _ completionHandler: (Bool) -> Void) {
        CacheManager.sharedInstance.removeDocumentFromCache(documentId: documentMetadata.vaultDocumentId, vault: viewModel.vault)
        completionHandler(true)
    }
}
