/*! @file DocumentsListViewModel.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

import Foundation
import VaultMobileSDK

class DocumentsListViewModel {
    var model : DocumentsListModel!
    
    var vault : VEVVault {
        get{
            return model.vault
        }
        set {
            model.vault = newValue
        }
    }
    
    var serverQuery : VEVVaultServerRequest? {
        get {
            return model.vaultServerQuery
        }
        set {
            model.vaultServerQuery = newValue
        }
    }
    
    var localQuery : VEVVaultLocalQuery? {
        get {
            return model.vaultLocalQuery
        }
        set {
            model.vaultLocalQuery = newValue
        }
    }
    
    var currentTabBarSelection : VaultTabBarItem {
        get {
            return model.currentTabBarSelection
        }
        set {
            model.currentTabBarSelection = newValue
        }
    }
    
    var currentMode : VEVVaultDocumentsListMode {
        get {
            return model.currentMode
        }
        set {
            model.currentMode = newValue
        }
    }
    
    init(model : DocumentsListModel){
        self.model = model
    }
    
    func clearResponses(){
        model.responses.removeAll()
    }
    
    func numberOfResponses(section : Int) -> Int {
        if model.responses.count == 0 {
            return model.showLoadingCell ? 1 : 0
        } else {
            return model.responses.count + (model.showLoadingCell ? 1 : 0)
        }
    }
    
    func shouldAttemptToFetchNewDocumentsFromVault(indexPath : IndexPath) -> Bool {
        return (getDocumentDataForResponseAtRow(row: indexPath.row) == nil) &&  currentMode == .ServerMode 
    }

    func getDocumentDataForResponseAtRow(row : Int) -> DocumentMetadata? {
        guard model.responses.indices.contains(row), let document = model.responses[row] as? Dictionary<String, Any> else {
            return nil
        }
        return DocumentMetadata(document: document)
    }
    
    func performSearch(searchText : String?, completion : @escaping ((Error?) -> ())){
        if currentMode == .ServerMode {
            createAndPerformServerSearch(searchText, completion)
        } else {
            createAndPerformLocalSearch(searchText, completion)
        }
    }

    func updateCurrentView(tabBarItem : VaultTabBarItem) {
        currentTabBarSelection = tabBarItem
        switch tabBarItem {
            case VaultTabBarItem.Browse:
                serverQuery = VEVVaultServerDocumentsSearchRequest(keywords: nil, vault: vault, parameters: ["limit": 20, "start": 0])
                localQuery = VEVVaultLocalQuery.create(withExpression:  nil)
            case VaultTabBarItem.Recents:
                serverQuery = VEVVaultServerDocumentsViewRequest(view: .recentDocuments, vault: vault, parameters:  ["limit": 20, "start": 0])
            case VaultTabBarItem.MyDocuments:
                serverQuery = VEVVaultServerDocumentsViewRequest(view: .myDocuments, vault: vault, parameters:  ["limit": 20, "start": 0])
            case VaultTabBarItem.Favorites:
                serverQuery = VEVVaultServerDocumentsViewRequest(view: .favorites, vault: vault, parameters:  ["limit": 20, "start": 0])
        }
    }

}

//Server Queries
extension DocumentsListViewModel {
    
    fileprivate func createAndPerformServerSearch(_ searchText: String?, _ completion: @escaping ((Error?) -> ())) {
        let keywords = searchText?.components(separatedBy: " ")
        serverQuery = VEVVaultServerDocumentsSearchRequest(keywords: keywords, vault: vault, parameters: ["limit": 20, "start": 0])
        executeCurrentServerQuery(completion: completion)
    }
    
    fileprivate func handleServerQueryResult(vaultQuery : VEVVaultServerRequest, response : [AnyHashable : Any]?, error : Error?, completion : @escaping ((Error?) -> ()) ){
        if isMismatchedQuery(vaultQuery: vaultQuery) {
            completion(NSError(domain: kServerQueryErrorDomain, code: ServerQueryError.queryMismatch, userInfo: nil))
            return
        } else if let response = response, error == nil {
            handleValidServerQueryResponse(response, vaultQuery)
            completion(nil)
        } else {
            model.showLoadingCell = false
            completion(error)
        }
    }
    
    fileprivate func isMismatchedQuery(vaultQuery : VEVVaultServerRequest) -> (Bool) {
        return (!(vaultQuery == serverQuery) || currentMode == .SavedLocally)
    }
    
    func executeCurrentServerQuery( completion : @escaping ((Error?) -> ())){
        UIApplication.shared.isNetworkActivityIndicatorVisible = true
        guard let vaultQuery = serverQuery else {
            completion(NSError(domain: kServerQueryErrorDomain, code: ServerQueryError.invalidQueryObject, userInfo: nil))
            return
        }
        
        if model.currentlyExecutingQuery == vaultQuery {
            return
        }
        model.currentlyExecutingQuery = vaultQuery
        runQuery(vaultQuery) { [weak self] (response, error) in
            self?.model.currentlyExecutingQuery = nil
            self?.model.showLoadingCell = true
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
            self?.handleServerQueryResult(vaultQuery: vaultQuery, response: response, error: error, completion: completion)
        }
    }
    
    @objc func runQuery(_ vaultQuery: VEVVaultServerRequest, _ completion: @escaping (VEVVaultAPIResponseHandler)) {
        VaultAPI.query(vaultQuery, completeHandler : completion)
    }
    
    fileprivate func handleValidServerQueryResponse(_ response: [AnyHashable : Any], _ vaultQuery : VEVVaultServerRequest) {
        let documents = response["documents"] as? [[AnyHashable : Any]]
        setServerResponse(documents, query: vaultQuery)
        if documents?.count == 0 {
            model.showLoadingCell = false
        }
    }
    
    fileprivate func setServerResponse(_ documentsResponse: [[AnyHashable : Any]]?, query : VEVVaultServerRequest?) {
        clearResponsesIfFirstQuery(query)
        guard let documentsResponse = documentsResponse else { return }
        extractIndividualDocumentsFromResponse(documentsResponse)
    }
    
    fileprivate func extractIndividualDocumentsFromResponse(_ documentsResponse: [[AnyHashable : Any]]) {
        for documentResponse in documentsResponse {
            if let unwrappedDocument = documentResponse["document"] as? [AnyHashable : Any] {
                model.responses.append(unwrappedDocument)
            }
        }
    }
    
    fileprivate func clearResponsesIfFirstQuery(_ query: VEVVaultServerRequest?) {
        if let start = query?.parameters?["start"] as? Int {
            if start == 0 {
                model.responses.removeAll()
            }
        } else {
            model.responses.removeAll()
        }
    }
}

//Local Queries
extension DocumentsListViewModel {
    
    fileprivate func createAndPerformLocalSearch(_ searchText: String?, _ completion: @escaping ((Error?) -> ())) {
        if let searchText = searchText {
            localQuery = VEVVaultLocalQuery.create(withExpression:  [VEVVaultQueryCriteria(field: CacheManager.sharedInstance.nameField.fieldName, like: searchText)])
        } else {
            localQuery = VEVVaultLocalQuery.create(withExpression:  nil)
        }
        executeCurrentLocalQuery(completion: completion)
    }
    
    func executeCurrentLocalQuery( completion : @escaping ((Error?) -> ())){
        guard let vaultQuery = localQuery else {
            completion( NSError(domain: kLocalQueryErrorDomain, code: LocalQueryError.invalidQueryObject, userInfo: nil))
            return
        }
        CacheManager.sharedInstance.query(localQuery: vaultQuery, vault: vault) { [weak self] (entries, error) in
            self?.handleLocalQueryResult(entries, error, completion: completion)
        }
    }
    
    fileprivate func handleLocalQueryResult(_ entries: [VEVVaultJSONCacheEntry]?, _ error: Error? , completion : @escaping (( Error?) -> ())) {
        if let entries = entries {
            model.showLoadingCell = false
            setLocalResponse(entries: entries)
            completion(nil)
        } else {
            if let error = error {
                print("An error occurred while refreshing local query \(error.localizedDescription)")
                completion(error)
            } else {
                completion( NSError(domain: kLocalQueryErrorDomain, code: LocalQueryError.unknownError, userInfo: nil))
            }
        }
    }
    
    fileprivate func setLocalResponse(entries :  [VEVVaultJSONCacheEntry]){
        for e in entries {
            model.responses.append(e.json)
        }
    }
    
}

