/*! @file VaultsVC.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

import UIKit
import VaultMobileSDK

class VaultsVC: UIViewController, UITableViewDataSource, UITableViewDelegate, UISearchBarDelegate {
    @IBOutlet weak var vaultSearch: UISearchBar?
    @IBOutlet weak var vaultsTableView: UITableView!
    var viewModel: VaultsVM!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupNavigationBarItems()
        setupVaultsTableView()
        setupVaultsSearch()
        loadInVaults()
    }

    fileprivate func setupVaultsTableView(){
        setupTableViewDelegates()
        configureTableView()
    }
    
    fileprivate func configureTableView() {
        vaultsTableView.tableFooterView = UIView()
        vaultsTableView.register(UINib(nibName:"VaultsCell", bundle: nil), forCellReuseIdentifier: "VaultsCell")
    }
    
    fileprivate func setupTableViewDelegates() {
        vaultsTableView.delegate = self
        vaultsTableView.dataSource = self
    }
    
    fileprivate func setupVaultsSearch() {
        vaultSearch?.placeholder = "Search"
        vaultSearch?.showsCancelButton = false
        vaultSearch?.delegate = self
    }
    
    fileprivate func loadInVaults(){
        vaultSearch?.text = ""
        viewModel.refreshCachedVaults()
        if let error = viewModel.getStringDescribingCurrentVaultChange() {
            showAlertDialog(message: error, title: "Warning", completion: nil)
        }
        vaultsTableView.reloadData()
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int)->Int{
        return viewModel.displayVaults.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath)->UITableViewCell{
        return setupVaultCell(tableView, indexPath)
    }
    
    fileprivate func setupVaultCell(_ tableView: UITableView, _ indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "VaultsCell") as! VaultsCell
        let selectedVault = viewModel.displayVaults[indexPath.row]
        
        setNameAndColorOfCell(selectedVault, cell)
        setCheckmarkOnSelectedVault(selectedVault, cell)
        return cell
    }
    
    fileprivate func setNameAndColorOfCell(_ selectedVault: VEVVault, _ cell: VaultsCell){
        cell.textLabel?.text = selectedVault.name
        cell.tintColor = UIColor.veevaOrange
    }
    
    fileprivate func setCheckmarkOnSelectedVault(_ selectedVault: VEVVault, _ cell: VaultsCell) {
        if(selectedVault.id == viewModel.currentVaultId ){
            cell.accessoryType = UITableViewCellAccessoryType.checkmark
        }else{
            cell.accessoryType = UITableViewCellAccessoryType.none
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 43.5
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath){
        guard isIndexValid(indexPath) else {
            return
        }
        selectVaultAtIndexPath(indexPath)
    }
    
    fileprivate func isIndexValid(_ indexPath: IndexPath) -> Bool {
        return viewModel.displayVaults.indices.contains(indexPath.row)
    }
    
    fileprivate func selectVaultAtIndexPath(_ indexPath: IndexPath) {
        let cell = vaultsTableView.cellForRow(at: indexPath)
        cell?.accessoryType = UITableViewCellAccessoryType.checkmark
        viewModel.updateCurrentlySelectedVault(indexPath: indexPath)
        vaultsTableView.reloadData()
    }
    
    func tableView(_ tableView: UITableView, didDeselectRowAt indexPath: IndexPath){
        let cell = vaultsTableView.cellForRow(at: indexPath)
        cell?.accessoryType = UITableViewCellAccessoryType.none
    }
    
    private func setupNavigationBarItems(){
        setupNavigationTitle()
        setupRefreshButton()
        setupDismissButton()
    }
    
    private func setupNavigationTitle() {
        title = "Vault"
        navigationItem.accessibilityLabel = "VaultsTitle"
    }
    
    private func setupRefreshButton() {
        navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Refresh", style: .done, target: self, action: #selector(VaultsVC.refreshPressed))
        navigationItem.rightBarButtonItem?.tintColor = UIColor.veevaOrange
    }
    
    @objc func refreshPressed(){
        let indicator = ProgressIndicator.addProgressIndicator(toView: view)
        VEVVaultAuth.sharedInstance().refreshUser { (_, error) in
            ProgressIndicator.removeProgressIndicator(indicator)
            self.handleRefresh(error)
        }
    }
    
    fileprivate func handleRefresh(_ error: Error?) {
        if let error = error {
            let sdkError = error as NSError
            showAlertDialog(message:  "Vault Response: " + sdkError.localizedDescription , title: "Failed to refresh", completion: nil)
        }else {
            loadInVaults()
        }
    }
    
    private func setupDismissButton() {
        navigationItem.leftBarButtonItem = UIBarButtonItem(title: "Dismiss", style: .done, target: self, action: #selector(VaultsVC.dismissPressed))
        navigationItem.leftBarButtonItem?.tintColor = UIColor.veevaOrange
    }
    
    @objc func dismissPressed(){
        dismiss(animated: true, completion: nil)
    }

    func showAlertDialog(message : String, title : String,  completion : ((Bool) -> Void)? = nil) {
        let alert = UIAlertController(title: title, message: message, preferredStyle: .alert)
        addOkAction(alert, completion)
        present(alert, animated: true, completion: nil)
    }
    
    fileprivate func addOkAction(_ alert: UIAlertController, _ completion: ((Bool) -> Void)?) {
        alert.addAction(UIAlertAction(title: "OK", style: .default , handler: { (action) in
            alert.dismiss(animated: false, completion: nil)
            completion?(true)
        }))
    }

    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        viewModel.displayVaults = viewModel.filterSearch(vaultData: viewModel.cachedVaults, searchText: searchText)
        vaultsTableView.reloadData()
    }
}
