/*! @file CacheManagerTest.swift
 *
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

import XCTest
import VaultMobileSDK
@testable import DocumentViewer

class CacheManagerTest: DocumentViewerTest {
    
    func testUserDirectory() {
        let username = "testusername"
        let pathToUserDirectory = CacheManager.getUserDirectory(username : username)
        XCTAssert(pathToUserDirectory.lastPathComponent == username)
    }
    
    func testSavedDocumentsDirectory(){
        let username = "testusername"
        let pathToSavedDocumentsDirectory = CacheManager.getSavedDocumentsDirectory(username: username)
        XCTAssert(pathToSavedDocumentsDirectory.lastPathComponent == "SavedDocuments")
    }
    
    func testGetDocumentFileURL(){
        let username = "testusername"
        let savedDocDir = CacheManager.getSavedDocumentsDirectory(username: username )
        let docID = 0
        let vault = VEVVault(id: 1, name: "testVault", url: "testurl")
        let expectedURL = savedDocDir.appendingPathComponent("document\(0)v\(1).pdf")
        let user = VEVUser(username: username, vaults: nil, userId: 0)
        XCTAssert(expectedURL == CacheManager.getDocumentFileURL(docID: docID, vault: vault, user: user))
    }
    
    func testIsDocumentFileCached(){
        let docID = 0
        let vault = VEVVault(id: 1, name: "testVault", url: "testurl")
        let username = "testusername"
        let user = VEVUser(username: username, vaults: nil, userId: 0)
        let _ = CacheManager.deleteDocumentFile(docId: docID, vault: vault, user: user)
        XCTAssertFalse(CacheManager.sharedInstance.isDocumentFileCached(documentId: docID, vault: vault, user: user))
        createTestFile(docID ,vault, user)
        XCTAssert(CacheManager.sharedInstance.isDocumentFileCached(documentId: docID, vault: vault, user: user))
        let _ = CacheManager.deleteDocumentFile(docId: docID, vault: vault, user: user)
    }
    
    func testCreateCacheDirectory(){
        let username = "testusername"
        let pathToSavedDocumentsDirectory = CacheManager.getSavedDocumentsDirectory(username: username)
        var bool : ObjCBool = false
        try? FileManager.default.removeItem(atPath: pathToSavedDocumentsDirectory.path)
        FileManager.default.fileExists(atPath: pathToSavedDocumentsDirectory.path, isDirectory: &bool)
        XCTAssert(bool.boolValue == false)
        CacheManager.sharedInstance.createCacheDirectory(username: username)
        FileManager.default.fileExists(atPath: pathToSavedDocumentsDirectory.path, isDirectory: &bool)
        XCTAssert(bool.boolValue == true)
    }
    
    func testCachedDocumentInterface(){
        let docID = 0
        let vault = VEVVault(id: 1, name: "testVault", url: "testurl")
        let testResponse = ["id" : 1] as [String : Any]
        let documentMetatdata = DocumentMetadata(document: testResponse)
        let expectation = XCTestExpectation(description: "testExp")
        CacheManager.sharedInstance.cacheType = MockFailureCache.self
        CacheManager.sharedInstance.cacheDocumentResponse(documentId: docID, vault: vault, documentMetadata: documentMetatdata, error: nil)
        CacheManager.sharedInstance.getCachedDocument(documentId: docID, vault: vault) { (entries, error) in
            XCTAssertNotNil(error)
            XCTAssertNil(entries)
            expectation.fulfill()
        }
        wait(for: [expectation], timeout: 20)
    }
    
    func testQueryDocumentInterface(){
        let vault = VEVVault(id: 1, name: "testVault", url: "testurl")
        let expectation = XCTestExpectation(description: "testExp")
        let localQuery = VEVVaultLocalQuery()
        CacheManager.sharedInstance.cacheType = MockFailureCache.self
        CacheManager.sharedInstance.query(localQuery:localQuery , vault: vault) { (entries, error) in
            XCTAssertNotNil(error)
            XCTAssertNil(entries)
            expectation.fulfill()
        }
        wait(for: [expectation], timeout: 20)
    }
    

    
    func testRemoveDocumentFromCache(){
        let vault = VEVVault(id: 1, name: "testVault", url: "testurl")
        let expectation = XCTestExpectation(description: "testExp")
        
        let testResponse = ["id" : 1] as [String : Any]
        let documentMetatdata = DocumentMetadata(document: testResponse)
        let cacheManagerDelegate = CacheManagerDelegateTest(expectation: expectation)
        
        CacheManager.sharedInstance.delegate = cacheManagerDelegate
        
        CacheManager.sharedInstance.documentsCache[vault.id] = MockSuccessCache.open(withName: "test", vault: vault, fields: [VEVVaultJSONField("Test", type: .integer)])
        CacheManager.sharedInstance.cacheDocumentResponse(documentId: 1, vault: vault, documentMetadata: documentMetatdata, error: nil)
        let username = "testusername"
        let user = VEVUser(username: username, vaults: nil, userId: 0)
        
        createTestFile(1, vault, user)
        CacheManager.sharedInstance.removeDocumentFromCache(documentId: 1, vault: vault)
        wait(for: [expectation], timeout: 10)
    }
    
    func testDownloadAndCacheDocument(){
        let vault = VEVVault(id: 1, name: "testVault", url: "testurl")
        let expectation = XCTestExpectation(description: "testExp")
        let username = "testusername"
        let user = VEVUser(username: username, vaults: nil, userId: 0)
        
        let testResponse = ["id" : 1] as [String : Any]
        let documentMetatdata = DocumentMetadata(document: testResponse)
        let cacheManagerDelegate = CacheManagerDelegateTest(expectation: expectation)
        let mockSuccessCache = MockSuccessCacheManager()
        mockSuccessCache.delegate = cacheManagerDelegate
        mockSuccessCache.documentsCache[vault.id] = MockSuccessCache.open(withName: "test", vault: vault, fields: [VEVVaultJSONField("Test", type: .integer)])
        mockSuccessCache.downloadAndCacheDocumentFile(vault, documentMetatdata, user)
            
    
        wait(for: [expectation], timeout: 10)
    }
    
    
}
