/*! @file VEVError.h
 *
 *  @copyright
 *  Copyright © 2017 Veeva Systems, Inc. All rights reserved.
 */
#import <Foundation/Foundation.h>


/*! @brief The error domain for NSErrors that may result during
 *  authorization.
 */
FOUNDATION_EXPORT NSString * const VEVAuthorizationErrorDomain;

/*! @brief The error domain for NSErrors resulting from Vault error
 *  conditions.
 *
 *  @see https://developer.veevavault.com/api/18.1/#errors
 */
FOUNDATION_EXPORT NSString * const VEVVaultErrorDomain;

/*! @brief Error domain for NSErrors resulting from a Vault exception.
 */
FOUNDATION_EXPORT NSString * const VEVVaultExceptionErrorDomain;

/*! @brief Error domain for NSErrors resulting from a Network problem.
 */
FOUNDATION_EXPORT NSString * const VEVNetworkErrorDomain;

/*! @brief Error domain for API usage errors.
 */
FOUNDATION_EXPORT NSString * const VEVAPIErrorDomain;

/*! @brief Error domain for encryption errors.
 */
FOUNDATION_EXPORT NSString * const VEVCryptographyErrorDomain;

/*! @brief Error domain for Cache errors
 */
FOUNDATION_EXPORT NSString * const VEVCacheErrorDomain;


/*! @brief Error codes that may result during authorization.
 */
typedef NS_ENUM(NSInteger, VEVAuthError) {
	/*! @brief Indicates invalid login credentials.
	 */
	VEVAuthInvalidLoginCredentialsError						= -1,

	/*! @brief Indicates the Vault session is invalid or expired.
	 */
	VEVAuthInvalidSessionError								= -2,

	/*! @brief Indicates insufficient permissions for the current
	 *  operations.
	 */
	VEVAuthInsufficientAccessError							= -3,

	/*! @brief Indicates the user is locked out.
	 */
	VEVAuthUserLockedOutError								= -4,

	/*! @brief Indicates the user canceled logging in.
	 */
	VEVAuthUserCanceledLogin								= -5,

	/*! @brief Indicates a misconfigured OpenID connect discovery
	 *  configuration.
     */
	VEVAuthInvalidSecurityProfileError						= -6,

	/*! @brief Indicates an OAuth security profile is configured
	 *  incorrectly.
	 */
	VEVAuthInvalidAuthenticationServerConfigurationError	= -7,

	/*! @brief Indicates user denied granting access to client
	 *  application.
	 */
	VEVAuthAccessDenied										= -8,

	/*! @brief Indicates an error occurred on the authentication
	 *  server.
	 */
	VEVAuthServerError										= -9,

	/*! @brief Indicates the authentication server is temporarily
	 *  unavailable.
	 */
	VEVAuthServerTemporarilyUnavailable						= -10,

	/*! @brief Indicates a client-side OAuth error occurred (e.g.,
	 *  because of a state mismatch or a misconfigured client, should
	 *  be treated as uncrecoverable.
	 */
	VEVAuthStateMismatchError								= -11,

	/*! @brief Indicates an HTTP error occurred during
	 *  authentications.
	 */
	VEVAuthHTTPError										= -12,

	/*! @brief Indicates an error occurred opening Safari during
	 *  authorization via OAuth.
	 */
	VEVAuthSafariOpenError									= -13,

	/*! @brief Indicates an error occurred serializing/deserializing
	 *  an authentication server response. This error implies a faulty
	 *  authentication server and should be treated as unrecoverable.
	 */
	VEVAuthJSONDeserializationError							= -14,

	/*! @brief Indicates an error occurred serializing/deserializing
	 *  an authentication server response. This error implies a faulty
	 *  authentication server and should be treated as unrecoverable.
	 */
	VEVAuthJSONSerializationError							= -15,

	/*! @brief Indicates an error occurred parsing an authorization
	 *  token or openid token. This error implies a faulty
	 *  authentication server and should be treated as unrecoverable.
	 */
	VEVAuthTokenParsingError								= -16,

	/*! @brief Indicates an error occurred refreshing access
	 *  tokens/open id tokens other than an expired refresh
	 *  token. This error implies a faulty authentication server and
	 *  should be treated as unrecoverable.
	 */
	VEVAuthTokenRefreshError								= -17,

	/*! @brief Indicates client-side values in the applications
	 *  Info.plist file are misconfigured.
	 */
	VEVAuthInvalidClientConfiguration						= -18,

	/*! @brief Indicates an unknown error occurred during
	 *  authorization via OAuth.
	 */
	VEVAuthUnknownOAuthError								= -19,

	/*! @brief Indicates the requested operation requires that the
	 *  user authenticate with Vault.
	 */
	VEVAuthUserNotLocallyAuthenticatedError					= -20,

	/*! @brief Indicates that the requested operation required a valid
	 *  Vault session.
	 */
	VEVAuthRemoteAuthenticationRequiredError				= -21
};

/*! @brief General error codes related to various Vault error conditions.
 *
 *  @see TODO VEVVaultServerErrorMessageKey.
 */
typedef NS_ENUM(NSInteger, VEVVaultError) {
	VEVVaultInvalidDataError				= -1,
	VEVVaultOperationNotAllowedError		= -2,
	VEVVaultParameterRequiredError			= -3,
	VEVVaultMaxFileSizeExceededError		= -4,
	VEVVaultDownForMaintenanceError			= -5,
	VEVVaultPasscodeChangeRequiredError		= -6,
	VEVVaultInactiveUserError				= -7,
	VEVVaultNoActiveVaultsError				= -8,
	VEVVaultSoleInactiveVaultError			= -9,
	VEVVaultConfigurationModeEnabledError	= -10,
	VEVVaultIPRangeError					= -11,
	VEVVaultSoleProtectedTemplateVaultError = -12,
	VEVVaultSelectedVaultUnavailableError	= -13,	// not a Vault API error response code but signaled by the SDK when a vault auth default logic does not select the actual Vault requested
    VEVVaultMethodNotSupportedError         = -14,
	VEVVaultInvalidTokenError				= -15,
	VEVVaultExpiredTokenError				= -16,
	VEVVaultInvalidConfigurationError		= -17,
	VEVVaultMalformedURLError				= -18,
	VEVVaultInvalidFilterError				= -19,
	VEVVaultInvalidParameterError           = -20,
	VEVVaultInvalidDocumentError			= -21
};

/*! @brief Error codes used if a Vault exception occurs.
 */
typedef NS_ENUM(NSInteger, VEVaultExceptionError) {
	VEVVaultExceptionUnspecifiedError				= -1,
	VEVVaultExceptionServerReturnedInvalidURLError	= -2
};

/*! @brief Error codes that may result during encryption or
 *  decryption.
 */
typedef NS_ENUM(NSInteger, VEVCryptError) {
	/*! @brief Indicates an unexpected error.
	 */
	VEVCryptUnspecifiedError	= -1,

	/*! @brief Indicates the user is not locally authenticated.
	 */
	VEVCryptUserLoggedOutError	= -2,

	/*! @brief Indicates an error occurred in the encryption process.
	 */
	VEVCryptEncryptionError		= -3,

	/*! @brief Indicates an error occurred in the decryption process.
	 */
	VEVCryptDecryptionError		= -4,

	/*! @brief Indicates an error occurred reading or saving the
	 *  encrypted file.
	 */
	VEVCryptFileIOError			= -5
};

/*! @brief Error codes that may result during cache
 *  operations.
 */
typedef NS_ENUM(NSInteger, VEVCacheError) {
    /*! @brief Indicates an unknown cache error.
     */
    VEVCacheUnknownError              = -1,
    
    /*! @brief Indicates the cache cannot be opened
     */
    VEVCacheUnableToOpenCacheError    = -2,
    
    /*! @brief Indicates there is no currently logged in user.
     */
    VEVCacheUserLoggedOutError        = -3,
    
    /*! @brief Indicates the requested caching operation failed.
     */
    VEVCacheRequestExceptionError     = -4
};

/*! @brief Error codes used if a network error occurs.
 */
typedef NS_ENUM(NSInteger, VEVNetworkError) {
    VEVNoNetworkConnection                = -1
};

/*!
 */
typedef NS_ENUM(NSInteger, VEVAPIError) {
	VEVInvalidParameter				= -1
};

/*! @brief The error key for the underlying Vault error message.
 */
FOUNDATION_EXPORT NSString * const VEVVaultServerErrorMessageKey;

/*! @brief The error key to retreive the Vault ID that resulted in a
 *  VEVVaultSelectedVaultUnavailableError.
 */
FOUNDATION_EXPORT NSString * const VEVVaultRequestedVaultIDKey;		// for VEVVaultSelectedVaultUnavailableError, userInfo key to retrieve ID of unavailable vault;

/*! @brief The error key for the underlying server response that
 *  resulted in a VEVAuthInvalidSecurityProfileError.
 */
FOUNDATION_EXPORT NSString * const VEVInvalidServerResponseKey;		// userInfo key to retrieve malformed response
