/*! @file VEVVaultCrypt.h
 *
 *  @copyright
 *  Copyright © 2017 Veeva Systems, Inc. All rights reserved.
 */
#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/*! @brief The block type used to call back to the application after
 *  performing encryption or decryption.
 *
 *  @param error The error object in the event an error occurred during
 *  the request.
 */
typedef void (^VEVVaultCryptResultHandler)(NSError * _Nullable error);

/*! @brief A singleton class that provides convenience methods for
 *  encrypting and decrypting locally stored files.
 *
 *  @note Encryption keys are generated per user, so the user must be
 *  \a locally \a authenticated to perform these operations. If the
 *  user is not locally authenticated an error is returned with error
 *  code \c VEVCryptUserLoggedOutError.
 */
@interface VEVVaultCrypt : NSObject

/*! @brief Return the singleton instance.
 */
+ (instancetype)sharedInstance;

/*! @brief Encrypts contents at \c path.
 *
 *  @remarks The user must be \a locally \a authenticated or an error
 *  with error code \c VEVCryptUserLoggedOutError is returned.
 *
 *  @remarks This method uses streams internally making it appropriate
 *  for files that do not fit comfortably in memory.
 *
 *  @param path The file path to encrypt.
 *
 *  @param attributes The file attributes.
 *
 *  @param handler The callback to invoke when complete.
 */
- (void)encryptFileAtPath:(NSString *)path
			   attributes:(nullable NSDictionary<NSString *,id> *)attributes
				  handler:(VEVVaultCryptResultHandler)handler;

/*! @brief Writes contents to disk after first encrypting them. This
 *  method is meant to mirror \c
 *  NSFileManager:createFileAtAtPath:contents:attributes: but with the
 *  addition of encryption.
 *
 *  @remarks The user must be \a locally \a authenticated or an error
 *  with error code \c VEVCryptUserLoggedOutError is returned.
 *
 *  @remarks This method is not appropriate for large files as it
 *  reads the entire contents at \c path into memory. For large files
 *  use VEVVaultCrypt:encryptFileAtPath:attributes:handler:
 *
 *  @param path The file path to write the new file to.
 *
 *  @param contents The data to encrypt and write.
 *
 *  @param attributes The file attributes.
 *
 *  @param error The error object in the event an error occurs writing
 *  the contents to disk.
 *
 *  @see VEVVaultAuth
 *  @see <a href="https://developer.apple.com/documentation/foundation/nsfilemanager/1410695-createfileatpath">NSFileManager:createFileAtPath:contents:attributes:</a>.
 */
- (BOOL)createEncryptedFileAtPath:(NSString *)path
						 contents:(NSData *)contents
					   attributes:(nullable NSDictionary<NSString *,id> *)attributes
							error:(NSError *_Nullable __autoreleasing *)error;

/*! @brief Returns an \c NSInputStream that decrypts the contents as
 *  they are read.
 *
 *  @remarks The user must be \a locally \a authenticated or an error
 *  with error code \c VEVCryptUserLoggedOutError is returned.
 *  
 *  @param srcPath The file path with the encrypted contents to
 *  decrypt.
 *
 *  @return The decrypting NSInputStream.
 */
- (nullable NSInputStream*)streamFromEncryptedFileAtPath:(NSString *)srcPath;

/*! @brief Returns the contents at path after first decrypting
 *  them. This method is meant to mirror \c
 *  NSFileManager:contentsAtPath: but with the addition of decryption.
 *
 *  @remarks The user must be \a locally \a authenticated or an error
 *  with code \c VEVCryptUserLoggedOutError is returned.
 *
 *  @remarks This method is not appropriate for large files as it
 *  reads the entire contents at \c path into memory. For large files
 *  use VEVVaultCrypt:streamFromEncryptedFileAtPath:
 *
 *  @param path The local file path of the file to read.
 *
 *  @param error The error object in the event an error occurs reading
 *  the contents from disk.
 *
 *  @return The encrypted contents.
 *
 *  @see VEVVautlCrypt::streamFromEncryptedFileAtPath:
 *  @see VEVVaultAuth
 *  @see <a href="https://developer.apple.com/documentation/foundation/nsfilemanager/1407347-contentsatpath">NSFileManager:contentsAtPath:</a>.  
 */
- (nullable NSData*)decryptContentsAtPath:(NSString *)path
									error:(NSError * _Nullable __autoreleasing *)error;

@end

NS_ASSUME_NONNULL_END
