/*! @file VEVSDKApplicationDelegate.h
 *
 *  @copyright
 *  Copyright © 2017 Veeva Systems, Inc. All rights reserved.
 */

#import <UIKit/UIKit.h>


/*! @brief A singleton object with an interface that corresponds to
 *  Apple's \c UIApplicationDelegate protocol. These methods must be
 *  called from within the host application's @c UIApplicationDelegate
 *  in order to integrate properly with VaultMobileSDK.
 */
@interface VEVSDKApplicationDelegate : NSObject


/*! @brief Return the singleton instance.
 */
+ (instancetype)sharedInstance;

/*! @brief Called from the application's @c UIApplicationDelegate @c
 *  application:didFinishLaunchingWithOptions: in order to to
 *  finalize VaultMobileSDK's initialization.
 *
 *  @param application The application object received by the app's @c
 *  UIApplicationDelegate.
 *
 *  @param launchOptions The launchOptions dictionary received by the
 *  app's @c UIApplicationDelegate.
 */
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions;

/*! @brief Called from the application's @c UIApplicationDelegate @c
 *  application:openURL:options: method to allow VaultMobileSDK to
 *  handle URL schemes that it is responsible for.
 *
 *  For example, this method must be invoked in order to
 *  properly enable authentication via OAuth.
 *
 *  @param application The application object received by the app's @c
 *  UIApplicationDelegate.
 *
 *  @param url The url object received by the app's @c
 *  UIApplicationDelegate.
 *
 *  @param options The options dictionary received by the app's @c
 *  UIApplicationDelegate.
 *  
 *  @return @c YES if the method handled the URL and no further
 *  processing is necessary, otherwise @c NO.
 */
- (BOOL)application:(UIApplication *)application
            openURL:(NSURL *)url
            options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options;

/*! @brief Forwards @c completionHandler to VaultMobileSDK in order to
 *  finalize any background session tasks managed by VaultMobileSDK.
 *
 *  For example, this method must be invoked in order to
 *  properly handle background file uploads to Vault.
 *
 *  @param application The application object received by the app's @c
 *  UIApplicationDelegate.
 *
 *  @param identifier The identifier received by the app's @c
 *  UIApplicationDelegate. This method will only track and invoke @c
 *  completionHandler if @c identifier matches the value expected by
 *  VaultMobileSDK (@c com.veeva.VaultMobileSDK.backgroundsession).
 *
 *  @param completionHandler The completionHandler received by the
 *  app's @c UIApplicationDelegate.
 *
 *  @see <a href="https://developer.apple.com/documentation/uikit/uiapplicationdelegate/1622941-application">UIApplication:application:handleEventsForBackgroundURLSession:completionHandler:</a>
 *  @see <a href="https://developer.apple.com/library/content/documentation/iPhone/Conceptual/iPhoneOSProgrammingGuide/BackgroundExecution/BackgroundExecution.html">iPhone Programming Guide Background Execution</a>
 */
- (void)application:(UIApplication *)application handleEventsForBackgroundURLSession:(NSString *)identifier
  completionHandler:(void (^)(void))completionHandler;

@end
