/*! @file VEVVaultAPI.h
 *
 *  @copyright
 *  Copyright © 2017 Veeva Systems, Inc. All rights reserved.
 */

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class VEVVault;
@class VEVVaultServerQuery;
@class VEVVaultServerRequest;
#pragma mark - 

/*! @brief The HTTP method to use when invoking a request on Vault.
 */
typedef NS_ENUM(NSUInteger, VEVRequestMethod) {

	/*! @brief GET HTTP method.
	 */
    VEVRequestMethodGET		= 1,

	/*! @brief POST HTTP method.
	 */
    VEVRequestMethodPOST	= 2,

	/*! @brief PUT HTTP method.
	 */
    VEVRequestMethodPUT		= 3
};

/*! @brief The encoding used by a request's params.
 */
typedef NS_ENUM(NSUInteger, VEVParameterEncoding) {

	/*! @brief Specifies request parameters are encoded with
	 *  ContentType application/x-www-form-urlencoded.
	 */
	VEVFormURLEncoding	= 1,

	/*! @brief Specifies request parameters are encoded with
	 *  ContentType application/json.
	 */
	VEVJSONEncoding		= 2
};

/*! @brief The block type used to call back to the application after a
 *  request to Vault.
 *
 *  @param response The vault JSON response serialized to an
 *  NSDictionary object.
 *
 *  @param error The error object in the event an error occurred during
 *  the request.
 */
typedef void (^VEVVaultAPIResponseHandler)(NSDictionary* _Nullable response, NSError* _Nullable error);

/*! @brief The block type used to call back to the application after a
 *  multipart request upload to Vault completes.
 *
 *  @param response The vault JSON response for the upload serialized
 *  to an NSDictionary object.
 *
 *  @param fileURL The file URL of the local document that was
 *  uploaded.
 *
 *  @param username The username of the user who performed the upload.
 *
 *  @param error The error object in the event an error occurred during
 *  the request.
 */
typedef void (^VEVVaultAPIUploadResultHandler)(NSDictionary * _Nullable response,
											   NSURL * fileURL,
											   NSString * username,
											   NSError * _Nullable error);

/*! @brief The block type used to call back to the application after a
 *  download request to Vault completes.
 *
 *  @param response The vault JSON response for the download
 *  serialized to an NSDictionary object. This will only be set in the
 *  event a Vault server error occurs.
 *
 *  @param fileURL The file URL of the local document location that
 *  the file was downloaded to.
 *
 *  @param MIMEType The MIMEType of the downloaded file.
 *
 *  @param username The username of the user who performed the
 *  download.
 *
 *  @param error The error object in the event an error occurred during
 *  the request.
 */
typedef void (^VEVVaultAPIDownloadResultHandler)(NSDictionary * _Nullable response,
												 NSURL * fileURL,
												 NSString * _Nullable MIMEType,
												 NSString * username,
												 NSError * _Nullable error);

/*! @brief The block type to call back to the application after a
 *  chunk of data is uploaded or downloaded.
 *
 *  @param progress The NSProgress that indicates the percent
 *  currently complete.
 */
typedef void (^VEVVaultAPIProgressHandler)(NSProgress* progress);

#pragma mark - Documents

/*! @brief The scope to apply search keywords to when searching.
 */
typedef NS_ENUM(NSUInteger, VEVSearchScope) {

	/*! @brief Search the file contents only.
	 */
	VEVSearchScopeContents				= 1,

	/*! @brief Search both the contents and the document record
	 *  fields.
	 */
	VEVSearchScopeContentsAndFields	= 2
};

/*! The sort order to apply to search list results.
 */
typedef NS_ENUM(NSUInteger, VEVSearchResultsSortOrder) {

	/*! @brief Applies the default sort order for records being
	 *  searched.
	 */
	VEVSearchResultsSortOrderDefault	= 0,

	/*! @brief Sorts the results in ascending order.
	 */
	VEVSearchResultsSortOrderAscending	= 1,

	/*! @brief Sorts the results in descending order.
	 */
	VEVSearchResultsSortOrderDescending = 2
};

/*! @brief The block type to call back to the application after a new
 *  Vault document has been created, for example, after uploading an
 *  unclassified document.
 *
 *  @param response The vault JSON response serialized to an
 *  NSDictionary object.
 *
 *  @param error The error object in the event an error occurred
 *  creating the document.
 */
typedef void (^VEVVaultAPICreateDocumentResultHandler)(NSDictionary * _Nullable response,
													   NSError * _Nullable error);

/*! @brief The block type to call back to the application after an
 *  existing Vault document has been updated, for example, after
 *  uploading a PDF file to a placeholder document.
 *
 *  @param response The vault JSON response serialized to an
 *  NSDictionary object.
 *
 *  @param error The error object in the event an error occurred
 *  updating the document.
 */
typedef void (^VEVVaultAPIUpdateDocumentResultHandler)(NSDictionary * _Nullable response,
													   NSError * _Nullable error);

/*! @brief The block type to call back to the application after
 *  retrieving the document information from Vault.
 *
 *  @param response The vault JSON response serialized to an
 *  NSDictionary object.
 *
 *  @param error The error object in the event an error occurred while
 *  retrieving the document information.
 */
typedef void (^VEVVaultAPIGetDocumentResultHandler)(NSDictionary * _Nullable response,
													NSError * _Nullable error);

/*! @brief The block type to call back to the application after
 *  retrieving the list of document information from Vault.
 *
 *  @param response The vault JSON response serialized to an
 *  NSDictionary object.
 *
 *  @param error The error object in the event an error occurred while
 *  retrieving the document information.
 */
typedef void (^VEVVaultAPIGetDocumentsResultHandler)(NSDictionary * _Nullable response,
													 NSError * _Nullable error);

/*! @brief The block type to call back to the application after
 *  retrieving document type information from Vault.
 *
 *  @param response The Vault JSON response serialized to an NSDictionary object.
 *
 *  @param error The error object in the event an error occurred while
 *  retrieving the document type information.
 */
typedef void (^VEVVaultAPIGetDocumentTypeResultHandler)(NSDictionary * _Nullable response,
														NSError * _Nullable error);

/*! @brief The block type to call back to the application after
 *  retrieving a document's properties from Vault.
 *
 *  @param response The Vault JSON response serialized to an NSDictionary object.
 *
 *  @param error The error object in the event an error occurred while
 *  retrieving the document's properties.
 */
typedef void (^VEVVaultAPIGetDocumentPropertiesResultHandler)(NSDictionary * _Nullable response,
															  NSError * _Nullable error);
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdocumentation"
/*! @brief A singleton class that is used to interface to the Vault
 *  server and the methods here help to issue requests and process
 *  responses.
 *
 *  @details VEVVaultAPI provides both convenience methods that
 *  closely correspond to the Vault REST API Documents module as well
 *  as more general methods that can be used to access any arbitrary
 *  Vault REST API endpoint.
 *
 *  #### Background Uploads
 *
 *  Some of the VEVVaultAPI methods wrap Vault's document upload
 *  endpoints. These methods fully support iOS background uploading as
 *  described in <a
 *  href="https://developer.apple.com/library/content/documentation/Cocoa/Conceptual/URLLoadingSystem/URLLoadingSystem.html">URL
 *  Session Programming Guide</a>. In order to enable this in your
 *  application you must implement <a
 *  href="https://developer.apple.com/documentation/uikit/uiapplicationdelegate/1622941-application">
 *  UIApplicationDelegate:application:handleEventsForBackgroundURLSession:completionHandler:</a>
 *  and forward all parameters to
 *  VEVSDKApplicationDelegate::application:handleEventsForBackgroundURLSession:completionHandler:.
 *
 *  @see
 *  VEVSDKApplicationDelegate::application:handleEventsForBackgroundURLSession:completionHandler:
 *
 *  @see <a
 *  href="https://developer.apple.com/documentation/uikit/uiapplicationdelegate/1622941-application">
 *  UIApplicationDelegate:application:handleEventsForBackgroundURLSession:completionHandler:</a>
 *
 *  @see <a
 *  href="https://developer.apple.com/library/content/documentation/Cocoa/Conceptual/URLLoadingSystem/URLLoadingSystem.html">URL
 *  Session Programming Guide</a>.
 *
 *  @see https://developer.veevavault.com/docs/#rest-api
 */
#pragma clang diagnostic pop
@interface VEVVaultAPI : NSObject

/*! @brief Return the singleton instance.
 */
+ (instancetype)sharedInstance;

/*! @brief Sends a request to a specific Vault endpoint. This method
 *  can be used to invoke any REST endpoint generically. If no other
 *  wrapper or helper method exists for a specific REST endpoint, this
 *  method can be used.
 *
 *  @param endpoint The Vault endpoint to send the request to. The
 *  endpoint must only include the portion of the endpoint URL that
 *  identifies the endpoint itself, e.g., \c objects/documents rather
 *  than \c https://myvault.com/api/v18.2/objects/documents.
 *
 *  @param vault The specific Vault to send the request to.
 *
 *  @param method The HTTP request method used in the request.
 *
 *  @param params The parameters to be encoded and included with the
 *  request. Must either be an \c NSDictionary \c or an \c NSArray \c .
 *
 *  @param encoding The encoding used for params.
 *
 *  @param completionHandler The callback that is invoked after
 *  completing the request.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 */
- (void)requestVaultEndpoint:(NSString*)endpoint
                       vault:(VEVVault*)vault
                      method:(VEVRequestMethod)method
                  parameters:(nullable id)params
		   parameterEncoding:(VEVParameterEncoding)encoding
           completionHandler:(VEVVaultAPIResponseHandler _Nullable)completionHandler;

/*! @brief Issues a request with a file to a specific Vault endpoint.
 *
 *  @param endpoint The Vault endpoint to send the request to.
 *
 *  @param vault The specific Vault to send the request to.
 *
 *  @param fileURL The absolute file path of the local file to upload
 *  represented as a file URL. The maximum file size that can be
 *  uploaded is 256 MB.
 *
 *  @param isEncrypted Pass in \c YES if the local file has been
 *  encrypted with
 *  VEVVaultCrypt::createEncryptedFileAtPath:contents:attributes:error:,
 *  otherwise \c NO.
 *
 *  @param method The HTTP request method used in the request.
 *
 *  @param params The parameters to be encoded and included with the
 *  request. Must either be an \c NSDictionary \c or an \c NSArray \c .
 *
 *  @param progressHandler The callback that is invoked each time a
 *  chunk of data is uploaded to the Vault server.
 *
 *  @param completionHandler The callback that is invoked after
 *  completing the request.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 *  @see VEVVaultCrypt
 */
- (void)requestVaultMultipartFormEndpoint:(NSString*)endpoint
                                    vault:(VEVVault*)vault
                                  fileURL:(NSURL*)fileURL
							  isEncrypted:(BOOL)isEncrypted
                                   method:(VEVRequestMethod)method
                               parameters:(nullable id)params
                          progressHandler:(VEVVaultAPIProgressHandler _Nullable)progressHandler
                        completionHandler:(VEVVaultAPIUploadResultHandler _Nullable)completionHandler;

/*! @brief Issues a request to a Vault file download endpoint, e.g.,
 *  objects/documents/2468/renditions/viewable_rendition__v.
 *
 *  @param endpoint The Vault endpoint to send the request to.
 *  
 *  @param vault The specific Vault to send the request to.
 *
 *  @param fileURL The absolute file path of the local file location to down to.
 *
 *  @param encrypt Pass in \c YES in order to encrypt the the
 *  downloaded file with
 *  VEVVaultCrypt::createEncryptedFileAtPath:contents:attributes:error:,
 *  otherwise \c NO.
 *
 *  @param params The parameters to be encoded and included with the
 *  request. Must either be an \c NSDictionary \c or an \c NSArray \c .
 *
 *  @param progressHandler The callback that is invoked each time a
 *  chunk of data is downloaded from the Vault server. Currently this
 *  is unsupported but is reserved for future use.
 *
 *  @param completionHandler The callback that is invoked after
 *  completing the request.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 *  @see VEVVaultCrypt
 */
- (void)requestVaultDownloadFileEndpoint:(NSString*)endpoint
								   vault:(VEVVault*)vault
								 fileURL:(NSURL*)fileURL // nil means use default file name
								 encrypt:(BOOL)encrypt
							  parameters:(nullable id)params
						 progressHandler:(VEVVaultAPIProgressHandler _Nullable)progressHandler
					   completionHandler:(VEVVaultAPIDownloadResultHandler _Nullable)completionHandler;

/*! @brief Sets the background callback to be invoked in the event an
 *  upload completes following a crash and subsequent restart by iOS.
 *
 *  @see
 *  VEVSDKApplicationDelegate::application:handleEventsForBackgroundURLSession:completeHandler
 *  and NSURLSession Programming Guide.
 */
- (void)setBackgroundUploadResultHandler:(VEVVaultAPIUploadResultHandler)handler;

/*! @brief Sets the background callback to be invoked in the event a
 *  download completes following a crash and subsequent restart by
 *  iOS.
 *
 *  @see
 *  VEVSDKApplicationDelegate::application:handleEventsForBackgroundURLSession:completeHandler
 *  and NSURLSession Programming Guide.
 */
- (void)setBackgroundDownloadResultHandler:(VEVVaultAPIDownloadResultHandler)handler;

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdocumentation"
/*! @brief Creates a Vault document without any associated content
 *  file, e.g., a placeholder document.
 *
 *  @param fields The dictionary of field value pairs that specify the
 *  field values for the newly created Vault document object. See <a
 *  href="https://developer.veevavault.com/api/18.2/#create-single-document">Create
 *  Single Document</a> for details about what may be specified in \c
 *  fields.
 *
 *  @param vault The Vault to create the document object on.
 *
 *  @param cHandler The callback that is invoked after the document
 *  object is successfully created or fails. It is always invoked on
 *  the main thread.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 */
#pragma clang diagnostic pop

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdocumentation"
- (void)createDocumentWithFields:(NSDictionary *)fields
						   vault:(VEVVault *)vault
				 completeHandler:(nullable VEVVaultAPICreateDocumentResultHandler)cHandler;

/*! @brief Uploads a locally stored file to a new Vault document
 *  object.
 *
 *  @param fields The dictionary of field value pairs that specify the
 *  field values for the new Vault document object. See <a
 *  href="https://developer.veevavault.com/api/18.2/#create-single-document">Create
 *  Single Document</a> for details about what may be specified in \c
 *  fields.
 *
 *  @param vault The Vault to create the document object on.
 *
 *  @param fileURL The absolute file path of the local file to upload
 *  represented as a file URL.
 *
 *  @param isEncrypted Pass in \c YES if the local file has been
 *  encrypted with
 *  VEVVaultCrypt::createEncryptedFileAtPath:contents:attributes:error:,
 *  otherwise \c NO.
 *
 *  @param pHandler The callback that is invoked each time a chunk of
 *  data is uploaded to the Vault server. It is always invoked on the
 *  main thread.
 *
 *  @param cHandler The callback that is invoked after the document
 *  object is successfully created or fails. It is always invoked on
 *  the main thread.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 *  @see VEVVaultCrypt
 */
#pragma clang diagnostic pop

- (void)createDocumentWithFields:(nullable NSDictionary *)fields
						   vault:(VEVVault *)vault
						 fileURL:(NSURL *)fileURL
				 fileIsEncrypted:(BOOL)isEncrypted
				 progressHandler:(nullable VEVVaultAPIProgressHandler)pHandler
				 completeHandler:(nullable VEVVaultAPICreateDocumentResultHandler)cHandler;

/*! @brief Updates a Vault document.
 *
 *  @param documentId The Vault document ID of the document.
 *
 *  @param fields The dictionary of field value pairs that specify the
 *  updated field values for the Vault document object.
 *
 *  @param vault The Vault on which the document resides.
 *
 *  @param cHandler The callback that is invoked after the document
 *  object is successfully updated or fails. It is always invoked on
 *  the main thread.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 */
- (void)updateDocument:(NSUInteger)documentId
				fields:(NSDictionary *)fields
				 vault:(VEVVault *)vault
	   completeHandler:(nullable VEVVaultAPIUpdateDocumentResultHandler)cHandler;

/*! @brief Uploads a locally stored file to an existing Vault document
 *  object, e.g., a placeholder document.
 *
 *  @param documentId The Vault document ID of the document.
 *
 *  @param fields The dictionary of field value pairs that specify the
 *  updated field values for the Vault document object.
 *
 *  @param vault The Vault on which the document resides.
 *
 *  @param fileURL The absolute file path of the local file to upload
 *  represented as a file URL.
 *
 *  @param isEncrypted Pass in \c YES if the local file has been
 *  encrypted with
 *  VEVVaultCrypt::createEncryptedFileAtPath:contents:attributes:error:,
 *  otherwise \c NO.
 *
 *  @param pHandler The callback that is invoked each time a chunk of
 *  data is uploaded to the Vault server. It is always invoked on the
 *  main thread.
 *
 *  @param cHandler The callback that is invoked after the document
 *  object is successfully updated or fails. It is always invoked on
 *  the main thread.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 *  @see VEVVaultCrypt
 */
- (void)updateDocument:(NSUInteger)documentId
				fields:(nullable NSDictionary *)fields
				 vault:(VEVVault *)vault
			   fileURL:(NSURL *)fileURL
	   fileIsEncrypted:(BOOL)isEncrypted
	   progressHandler:(nullable VEVVaultAPIProgressHandler)pHandler
	   completeHandler:(nullable VEVVaultAPIUpdateDocumentResultHandler)cHandler;

/*! @brief Retrieves document metadata for the specified document on
 *  Vault.
 *
 *  @param documentId The Vault document ID of the document.
 *
 *  @param vault The Vault where the document exists.
 *
 *  @param cHandler The callback that is invoked after retrieving the
 *  document information from Vault.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 */
- (void)getDocument:(NSUInteger)documentId
              vault:(VEVVault*)vault
    completeHandler:(VEVVaultAPIGetDocumentResultHandler)cHandler;

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdocumentation"
/*! @brief Retrieves document metadata for the specified document
    type.
 *
 *  @param name The name of the document type on Vault. \c name must
 *  be one of the top-level of the document
 *  type/subtype/classification hierarchy. See <a
 *  href="https://developer.veevavault.com/api/18.2/#https://developer.veevavault.com/api/18.2/#retrieve-document-type">Retrieve
 *  Document Type</a>.
 *
 *  @param vault The Vault where the document type exists.
 *
 *  @param cHandler The callback that is invoked after
 *  retrieving the document type metadata from Vault.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 *  @see 
 */
#pragma clang diagnostic pop
-(void)getDocumentType:(NSString *)name
                 vault:(VEVVault *)vault
       completeHandler:(VEVVaultAPIGetDocumentTypeResultHandler)cHandler;

/*! @brief Retrieves document properties for the specified document
 *  on Vault.
 *
 *  @param documentId The Vault document ID of the document.
 *
 *  @param vault The Vault where the document exists.
 *
 *  @param cHandler The callback that is invoked after retrieving the
 *  document properties from Vault.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 */
- (void)getDocumentProperties:(NSUInteger)documentId
                        vault:(VEVVault *)vault
              completeHandler:(VEVVaultAPIGetDocumentPropertiesResultHandler)cHandler;

/*! @brief Executes a VEVVaultServerQuery on a Vault.
 *
 *  @param request The request to execute.
 *
 *  @param cHandler The callback that is invoked after executing the
 *  query.
 *
 *  @remarks Note that calling this method will prompt the user to
 *  authenticate if necessary.
 *
 *  @see VEVVaultAuth
 */
- (void)query:(VEVVaultServerRequest *)request completeHandler:(VEVVaultAPIResponseHandler)cHandler;


@end

NS_ASSUME_NONNULL_END
