/*! @file VEVVaultAuth.h
 *
 *  @copyright
 *  Copyright © 2017 Veeva Systems, Inc. All rights reserved.
 */

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h> // OAuth needs UIViewController


NS_ASSUME_NONNULL_BEGIN

/*! @brief An object type that refers to a particular
 * Vault. VEVVaultAuth::currentUser tracks the authenticated user's
 * list of available vaults and several methods throughout
 * VaultMobileSDK require a VEVVault parameter as a means of
 * specifying a Vault to apply operations to.
 *
 * @see VEVUser::vaults.
 * @see VEVVaultAuth::currentUser.
 */
@interface VEVVault : NSObject <NSCoding>

/*! @brief The Vault ID.
 */
@property (nonatomic) NSUInteger Id;

/*! @brief The Vault name.
 */
@property (copy, nonatomic) NSString* name;

/*! @brief The Vault DNS.
 */
@property (copy, nonatomic) NSURL* URL;

- (instancetype)initWithId:(NSUInteger)Id name:(NSString*)name url:(NSString*)url;

@end

/*! @brief An object type that refers to a Vault user. This is used
 *  primarily to track information about the currently authenticated
 *  user.
 *
 *  @see VEVVaultAuth::currentUser.
 */
@interface VEVUser : NSObject <NSCoding>

/*! @brief The user's username.
 */
@property (strong, nonatomic, readonly) NSString* username;

/*! @brief The user's list of available Vaults.
 */
@property (strong, nonatomic, readonly, nullable) NSArray<VEVVault*>* vaults;

/*! @brief The user's user ID.
 */
@property (nonatomic, readonly) NSUInteger userId;

- (instancetype)initWithUsername:(NSString*)username vaults:(nullable NSArray<VEVVault*>*)vaults userId:(NSUInteger)userId;

@end


/*! @brief The block type used to call back to the application after a
 *  login attempt has succeeded or failed.
 *
 *  @param user The current user object if authenticated, or nil.
 *
 *  @param error The error object in the event an error occurred during
 *  login.
 *
 *  @see VEVVaultAuth:authenticateWithHandler.
 */
typedef void (^VEVVaultAuthLoginResultHandler)(VEVUser * _Nullable user,
											   NSError * _Nullable error);

/*! @brief The block type used to call back to the application when
 *  refreshing the current user's information from Vault.
 *
 *  @param user The user object representing the current user or nil
 *  if the user has logged out.
 *
 *  @param error The error object in the event an error occurred
 *  refreshing the current user's vaults.
 */
typedef void (^VEVVaultAuthRefreshUserResultHandler)(VEVUser * _Nullable user,
													 NSError * _Nullable error);

/*! @brief The block type used to call back to the application when
 *  the user is logged out.
 *
 *  @param user The user object representing the user that was
 *  logged out.
 */
typedef void (^VEVVaultAuthLogoutHandler)(VEVUser * user);

/*! @brief The delegate to notify of events sent from \c
 *  VEVVaultAuth's lock screen view.
 *
 *  The SDK has the ability to present a lock screen when the mobile
 *  app returns from the background. In order to process the events
 *  from that lock screen the mobile app must implement this delegate.
 */
@protocol VEVPasscodeManagerDelegate <NSObject>

@optional

/*! @brief Informs the delegate when the user has unlocked the lock
 *  screen.
 */
- (void)unlockWasSuccessful;
@end

/*! @brief A singleton class that manages Vault authentication.
 *
 *  @details Many of the features of VaultMobileSDK require
 *  authentication with Vault. Authentication can be performed in one
 *  of two ways: either \a implicitly or \a explicitly. An \a implicit
 *  login occurs whenever a VaultMobileSDK method is called that
 *  requires authentication—that is, a valid session ID, and either
 *  the user has not logged in previously or the current session ID
 *  has expired. When an implicit login occurs, the method that
 *  triggered it is interrupted and VaultMobileSDK displays a
 *  full-screen login UI (see \a login \a UI below). Once logged in
 *  the triggering method resumes. Any errors that occur are reported
 *  back to the application by means of callbacks. Implementations for
 *  these callbacks are the only code that the application developer
 *  needs to write in order to support authentication.
 *
 *  In addition to implicit login, the application developer can force
 *  authentication (or re-authentication) at any time by calling
 *  VEVVaultAuth::authenticateWithHandler:. This is termed \a explicit
 *  login.
 *
 *  \b Login \b UI
 *
 *  When the login view is displayed, regardless of whether in
 *  response to an implicit login or an explicit login, it performs a
 *  \a refresh \a session operation or it performs an \a initial \a
 *  login. When performing a \a refresh \a session, the username is
 *  pre-filled and rendered read-only. In addition, a \b Cancel button
 *  is displayed that allows the user to dismiss the login UI and
 *  return to the host app without performing any authentication (see
 *  "Local Authentication and Offline Access" below). In contrast,
 *  when performing an \a initial \a login the username is blank and
 *  the login UI cannot be dismissed until authentication completes
 *  successfully.
 *
 *  \b Local \b Authentication \b and \b Offline \b Access
 *
 *  Once authentication is complete, VaultMobileSDK regards the user
 *  as \a locally \a authenticated. This simply means that by virtue
 *  of having once logged in to Vault and obtained a session ID, the
 *  user is considered authenticated with respect to client-side
 *  operations until the user explicitly logs out. An expired session
 *  ID has no bearing on this status. In practice, this means that the
 *  user can dismiss the login view without re-authenticating with
 *  Vault. This ensures that the user can still access protected
 *  features of the application that don't require direct interaction
 *  with Vault even if network access is unavailable.
 *
 *  \b Pin \b Code \b View
 *
 *  VaultMobileSDK provides a client-side pin code protected lock
 *  screen that displays automatically after the application launches
 *  or is brought into the foreground from the background. New users
 *  set their pin codes immediately after authenticating for the first
 *  time.
 *
 *  \b Biometric \b Authentication
 *
 * Biometric Authentication may be used when re-authenticating after a
 * Vault session ID has expired and when unlocking the lock screen.
 * Users have the option to enable Biometric Authentication by
 * checking the Biometric Authentication on the login view.
 *
 *  \b OAuth
 *
 *  VaultMobileSDK supports authentication via OAuth for Vaults that
 *  have been configured for OAuth support. Authentication via OAuth
 *  behaves identically to basic authentication with the following
 *  exceptions:
 *
 *  1. Once the user enters his or her username in the login view,
 *  VaultMobileSDK detects that the user is configured for OAuth and
 *  an \c SFSafariViewController is displayed with the IDP's login
 *  page loaded. Authentication via OAuth is conducted as normal and
 *  once complete, the application is notified via callback in the
 *  same manner as in basic authentication.
 *
 *  2. Re-authentication is performed automatically and invisibly via
 *  refresh token or, if the refresh token expires, the VaultMobileSDK
 *  \a refresh \a session login view is bypassed entirely and the user
 *  re-authenticates with the IDP directly from within \c
 *  SFSafariViewController.
 *
 *  3. Biometric Authentication is not supported for OAuth users.
 *
 *  The following steps are required in order to configure
 *  VaultMobileSDK for OAuth support. Once configured, VaultMobileSDK
 *  manages all of the particulars of OAuth on behalf of the host
 *  application.
 *
 *  1. Add \c com.veeva.oauth.client_id to the Info.plist file with
 *  the corresponding OAuth client ID configured on the authentication
 *  server.
 *
 *  2. Add \c com.veeva.oauth.redirect_uri to the Info.plist file with
 *  the corresponding redirect URI configured on the authentication
 *  server.
 *
 *  3. In the application's \c
 *  UIApplicationDelegate:application:openURL:options: method, invoke
 *  \c VEVSDKApplicationDelegate:application:openURL:options:.
 */
@interface VEVVaultAuth : NSObject

/*! @brief The user object that represents the locally authenticated
 *  user.
 */
@property (copy, nonatomic, nullable) VEVUser* currentUser;

/*! @brief The object that acts as the delegate for \c VEVVaultAuth's
 *  lock screen view.
 */
@property (weak, nonatomic) id<VEVPasscodeManagerDelegate> passcodeDelegate;

/*! @brief Return the singleton instance.
 */
+ (instancetype)sharedInstance;

/*! @brief Forces the appearance of the login view in order to perform
 *  either initial authentication or re-authentication to refresh the
 *  session.
 *
 *  @param handler The callback invoked after authentication
 *  completes.
 *
 *  @remarks When \c handler is invoked after a successful login, it
 *  passes back the user object for the current user. If
 *  authentication fails, details regarding the failure are contained
 *  in the error parameter. If an error occurs during initial login \c
 *  user will be \c nil . If an error occurs while re-authenticating,
 *  then \c user corresponds to the user returned when authentication
 *  last succeeded.
 */
- (void)authenticateWithHandler:(nullable VEVVaultAuthLoginResultHandler)handler;

/*! @brief Logs the user out and invokes the logout handler if
 *  set. The user is no longer locally authenticated after calling
 *  this method.
 */
- (void)logout;

/*! @brief Sets the callback handler that will be invoked when the
 *  user is logged out. Logout will occur when VEVVaultAuth::logout is
 *  called or may occur from within VaultMobileSDK itself. For
 *  example, when the user resets the pincode the user is logged out
 *  and prompted to log back in again.
 *
 *  @param handler The callback invoked after logout completes.
 */
- (void)setLogoutHandler:(nullable VEVVaultAuthLogoutHandler)handler;

/*! @brief Forces the lock screen to display immediately.
 *
 *  @return \c YES if the user is locally authenticated and the
 *  passcode displayed, otherwise \c NO.
 */
- (BOOL)showLockScreen;

/*! @brief Returns whether the lock screen is currently presented or not
 *
 *  @return \c YES if the lock screen is currently present otherwise \c NO.
 */
- (BOOL)isShowingLockScreen;

/*! @brief Synchronizes the state associated with the current user
 *  with Vault. Currently this only includes the user's list of
 *  available vaults.
 *
 *  @param handler The callback invoked upon completion.
 *
 *  @see VEVVaultAuth::authenticateWithHandler:
 */
- (void)refreshUserWithHandler:(nullable VEVVaultAuthRefreshUserResultHandler)handler;

/*! @brief Enables or disables Biometric Authentication for both
 *  VEVVaultSDK's login view and passcode view. This must be set after
 *  calling
 *  VEVSDKApplicationDelegate::application:didFinishLaunchingWithOptions:
 *  and before calling any method that may display the login view or
 *  passcode view.
 *
 *  @param enable YES to enable Biometric Authentication, NO to disable. By
 *  default this value is set to YES.
 *
 *  @remarks This is an opt-out method for overriding the Biometric
 *  Authentication configuration that may be set via EMM. If this
 *  value is NOT explicitly set via this method then VaultMobileSDK
 *  will use an EMM value located in the \c
 *  com.apple.configuration.managed EMM dictionary with key \c
 *  VeevaVaultBiometricAuthenticationVisible. If no EMM value is
 *  present it will default to YES.
 */
- (void)enableBiometricAuthentication:(BOOL)enable;

/*! @brief Reports whether Biometric Authentication enabled for
 *  VEVVaultMobileSDK's login view and passcode view.
 *
 *  @see VEVVaultAuth::enableBiometricAuthentication:
 *
 *  @return YES if Biometric Authentication is enabled, otherwise NO.
 */
- (BOOL)isBiometricAuthenticationEnabled;

@end

NS_ASSUME_NONNULL_END
