/*! @file VEVVaultJSONCache.h
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

#import <Foundation/Foundation.h>

#import "VEVVaultCache.h"


NS_ASSUME_NONNULL_BEGIN

@class VEVVaultJSONCacheEntry, VEVVaultJSONField, VEVVaultLocalQuery;

/*! @brief The block type used to call back to the application after
 *  cache methods are called.
 *
 *  @param error The error object in the event an error occurred
 *  during the cache operation.
 *
 */
typedef void (^VEVVaultJSONCacheReindexResultHandler)(NSError * _Nullable error);

/*! @brief The block type used to call back to the application after
 *  the query method has be invoked.
 *
 *  @param entries The cache entries resulting from the cache query.
 *
 *  @param error The error object in the event an error occurred
 *  during the query.
 *
 *  @see VEVVaultJSONCache::cache:vault:handler:.
 */
typedef void (^VEVVaultJSONCacheQueryResultHandler)(NSArray<VEVVaultJSONCacheEntry *>* _Nullable entries, NSError * _Nullable error);

/*! @brief The block type used to call back to the application after
 *  the cache method has be invoked.
 *
 *  @param entriesCached An array of the entries that were
 *  successfully cached.
 *  
 *  @param error The error object in the event an error occurred
 *  during the cache operation.
 *  
 *  @see VEVVaultJSONCache::query:handler:.
 */
typedef void (^VEVVaultJSONCacheCacheResponseResultHandler)(NSArray<VEVVaultJSONCacheEntry *> * _Nullable entriesCached, NSError * _Nullable error);

/*! @brief Instances of this class are used to store JSON responses
 *  from Vault for later use—for example when the user is
 *  offline. Caches support complex queries, are persistent, and retain
 *  their contents across application restarts.
 *
 *  #### Opening a Cache
 *
 *  You create or open an existing VEVVaultJSONCache by calling
 *  VEVVaultJSONCache::openCacheWithName:vault:fields:, passing in a
 *  name for the cache, a VEVVault, and a collection of
 *  VEVVaultJSONField objects. The name and the VEVVault together
 *  define a \a scope for the responses being stored (see \b Caching
 *  \b Responses). The VEVVaultJSONField objects define which keys
 *  from the response are searchable when retrieving stored responses
 *  (see \b Querying \b Responses).
 *
 *  While the name you give to a cache can be anything you like, best
 *  practice is to choose a name that corresponds to a Vault type such
 *  as 'DocumentCache'. Responses stored in the cache should then all
 *  correspond to this type.
 *
 *  Example:
 *
 *		if let vault = VEVVaultAuth.sharedInstance().currentUser?.vaults?.first {
 *             let nameField = VEVVaultJSONField("name__v", type: .text)
 *             let idField = VEVVaultJSONField("id", type: .integer)
 *             let cache = VEVVaultJSONCache.open(withName: "Documents", vault: vault, fields: [nameField, idField])
 *         }
 *
 *  #### Caching Responses
 *
 *  You cache responses by calling the
 *  VEVVaultJSONCache::cache:vault:handler: method. Each response in
 *  the given collection must have a key at the top level labeled
 *  "id". Caching a response will update any previously cached
 *  response that has a matching id, as long as the cache being used
 *  to update has the same \a scope as the original cache—that is, as
 *  long as it has the same name and Vault. Otherwise, it will be
 *  added as a new response. Cache scopes are enforced even when
 *  different cache instances are used, for example when recreating
 *  caches after restarting the application.
 *
 *  Example:
 *
 *		let sampleResponse : [Dictionary< String, Any >] = [[
 *                     "id" : 1,
 *                     "version_id":1_0_1,
 *                     "name__v" : "clinical document"
 *                     ],[
 *                     "id" : 2,
 *                     "version_id":2_0_1,
 *                     "name__v" : "site document"
 *                     ]]
 *             
 *             cache.cache(sampleResponse, vault: vault) { (entries, error) in
 *                 print("Newly added cache entries \(entries)")
 *              }
 *
 *  #### Querying Responses
 *
 *  You query a cache for responses by constructing a
 *  VEVVaultLocalQuery object from one or more VEVVaultQueryCriteria,
 *  and passing it to VEVVaultJSONCache::query:handler:. The process
 *  overall is meant to correspond closely with writing directly in a
 *  query language, as the following example demonstrates:
 *
 *		let query = VEVVaultLocalQuery.create(withExpression: [
 *			VEVVaultQueryCriteria(field: "name__v", like: "Pleasanton"),
 *			VEVVaultLogicalOperatorAnd,
 *			VEVVaultQueryCriteria(field: "pages__v", equalTo: @2)])
 *
 *		documentsJsonCache.query(localQuery, handler: { (entries, error) in
 *			print("JsonCacheEntries \(entries)")
 *		})
 *
 *  The fields given to the query object are limited to those that
 *  were specified when opening the cache.
 */
@interface VEVVaultJSONCache : VEVVaultCache

/*! @brief The fields that this cache has enabled for querying.
 */
@property (strong, nonatomic, readonly, nonnull) NSArray<VEVVaultJSONField*> * fields;

/*! @brief Opens or creates a VEVVaultJSONCache with the specified
 *  name, vault, and search fields. The first time a cache with a
 *  given name and vault is opened it is created. On subsequent calls
 *  with the same name and vault the existing cache is opened and returned.
 *
 *  The \c fields parameter specifies which keys in the cached JSON
 *  responses are searchable.
 *
 *  @param name The name of the cache to open.
 *  
 *  @param vault The Vault server from which all JSON responses in the
 *  cache originated from.
 *  
 *  @param fields The collection of searchable fields.
 */
+ (instancetype)openCacheWithName:(NSString *)name vault:(VEVVault *)vault fields:(NSArray<VEVVaultJSONField*> *)fields;

/*! @brief Caches a collection of JSON responses originating from a
 *  Vault API endpoint, e.g., documents. Each response is required to
 *  have a key at the top level labeled "id" which uniquely identifies
 *  it within the cache. If another response is subsequently cached
 *  with an id matching that of an existing response, then the
 *  original response will be replaced with the updated response.
 *
 *  @param jsonResponse An array of NSDictionaries containing JSON
 *  representations of Vault entities, e.g., Documents.
 *  
 *  @param vault The Vault from which \c jsonResponse originated.
 *  
 *  @param handler The callback invoked upon completion.
 *
 *  @remarks The vault that is passed in is meant to serve as a check
 *  that ensures all cached responses correspond to the same
 *  originating Vault. If \c vault does not match the cache's \c vault
 *  property and error is returned.
 */
- (void)cache:(NSArray<NSDictionary*> *)jsonResponse vault:(VEVVault *)vault handler:(VEVVaultJSONCacheCacheResponseResultHandler _Nullable)handler;

/*! @brief Queries the local cache and asynchronously returns the
 *  matching responses as an array of VEVVaultJSONCacheEntry objects.
 *
 *  @param query The VEVVaultLocalQuery to execute. Only field names
 *  that correspond to the \c fields property can be queried.
 *  
 *  @param handler The callback invoked upon completion.
 *
 *  @see VEVVaultJSONCache::openCacheWithName:vault:fields:  
 *  @see VEVVaultLocalQuery
 *  @see VEVVaultJSONCacheEntry
 */
- (void)query:(VEVVaultLocalQuery *)query handler:(VEVVaultJSONCacheQueryResultHandler _Nullable)handler;

/*! @brief Removes the specified entries from the cache.
 *  
 *  @param entries An array of VEVVaultCacheEntry to be removed.
 *  
 *  @param handler The callback invoked upon completion.
 */
- (void)removeEntries:(NSArray<VEVVaultCacheEntry *>*)entries handler:(VEVVaultCacheRemoveEntriesResultHandler _Nullable)handler;

/*! @brief Removes all entries from the cache.
 *
 *  @param handler The callback invoked upon completion.
 */
- (void)removeAllEntries:(VEVVaultCacheRemoveAllEntriesResultHandler _Nullable)handler;

/*! @brief Reindexes the cache entries. Calling this method is
 *  required when new fields that have been added to the JSON response
 *  schema need to be searchable.
 *
 *  @param newFields The updated list of searchable fields. The
 *  original list of fields is replaced completely by this updated
 *  list so it is important to include any original fields that should
 *  remain searchable.
 *
 *  @param handler The callback invoked upon completion.
 */
- (void)reindexWithFields:(NSArray<VEVVaultJSONField *> *)newFields handler:(VEVVaultJSONCacheReindexResultHandler _Nullable)handler;

@end
NS_ASSUME_NONNULL_END
