/*! @file VEVVaultJSONField.h
 *  @copyright
 *  Copyright © 2018 Veeva Systems, Inc. All rights reserved.
 */

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/*! @brief An enumeration that corresponds to basic JSON types, e.g.,
 *  \c VEVFieldTypeInteger -> \c number and \c VEVFieldTypeText ->
 *  \c string. Other types are not currently supported.
 */
typedef NS_ENUM(NSInteger, VEVFieldType) {

	/*! @brief Represents JSON number type.
	 */
    VEVFieldTypeInteger = 0,

	/*! @brief Represents JSON string type.
	 */
    VEVFieldTypeText    = 1
};

/*! @brief Instances of this class are associated with a specific
 *  key/value pair from a Vault response schema, e.g., the \c name__v
 *  key from the JSON schema used by the \c documents endpoint. It is
 *  used to specify which keys are searchable when creating a
 *  VEVVaultJSONCache. All such keys must exist at the top level of
 *  the JSON schema. The \c type property must correspond the
 *  appropriate JSON type in the JSON schema.
 */
@interface VEVVaultJSONField : NSObject

/*! @brief The name of the key from the JSON schema.
 */
@property (nonatomic, readonly) NSString* fieldName;

/*! @brief The type associated with the field in the Vault response.
 */
@property (nonatomic, readonly) VEVFieldType type;

/*! @brief Initializes a VEVField object with a key name and type.
 *
 *  @param fieldName The name of the key as it exists in a Vault
 *  response JSON schema.
 *  
 *  @param type The VEVFieldType of the value stored that matches the
 *  corresponding JSON schema type, e.g. VEVFieldTypeInteger -> JSON
 *  number.
 */
- (instancetype)init:(NSString *)fieldName type:(VEVFieldType)type;

/*! @brief Returns the value for the JSON response that matches key \c
 *  fieldName. Only top level JSON keys are searched. Create a
 *  subclass and override this method if the field value is a collection or if the key/value pair is nested further down from the top level.
 *
 *  @param json An NSDictionary representation of a JSON response
 *  originating from a Vault API endpoint.
 */
- (id)parse:(NSDictionary *)json;
@end

NS_ASSUME_NONNULL_END
